#include <vector>
double TransfoX(double X); 
void Linearisation(int RunStart, int RunStop){

    //========================================================================
    //-------------------Open tree and define Hist----------------------------
    //======================================================================== 

    TChain *ch = new TChain("PhysicsTree"); 


    for(int run = RunStart ; run <= RunStop ; run++){
        ch->Add(Form("../../root/analysis/linea_%d.root",run));
    } //End of run loop

    std::vector<TH1F*> PositionX,PositionY;
    for (int i=1; i<=4; i++ ){
        std::string PosXName = "hPositionXDet" + std::to_string(i) ;
        std::string PosXTitle = "PositionXDet" + std::to_string(i) ;
        std::string PosYName = "hPositionYDet" + std::to_string(i) ;
        std::string PosYTitle = "PositionYDet" + std::to_string(i) ;
        TH1F * PosX;
        TH1F * PosY;
        if ( i==1 ){
            PosX = new TH1F(PosXName.c_str(),PosXTitle.c_str(),1000,0,40);
            PosY = new TH1F(PosYName.c_str(),PosYTitle.c_str(),1000,0,60);
        }//End if det 1

        if ( i==2 ){
            PosX = new TH1F(PosXName.c_str(),PosXTitle.c_str(),1000,0,63);
            PosY = new TH1F(PosYName.c_str(),PosYTitle.c_str(),1000,0,92);

        }//End if det 2

        if ( i==3 || i==4 ) {
            PosX = new TH1F(PosXName.c_str(),PosXTitle.c_str(),1000,-1000,0);
            PosY = new TH1F(PosYName.c_str(),PosYTitle.c_str(),1000,0,160);
        }//End if det 3 or 4

        PositionX.push_back(PosX);
        PositionY.push_back(PosY);
    }//End of i loop
    TH1F *ZoomDet1 = new TH1F("ZoomDet1","ZoomDet1",10,19,20);
    TH1F *ZoomDet1Adjusted = new TH1F("ZoomDet1Adjusted","ZoomDet1Adjusted",10,19,20);

    //========================================================================
    //-----------------------Importing conditional var------------------------
    //========================================================================
    TFPMWPhysics *FPMW = new TFPMWPhysics();
    ch->SetBranchAddress("FPMW",&FPMW);


    //========================================================================
    //--------------------------TreatEvent------------------------------------
    //========================================================================
    int Nentries = ch->GetEntries();
    int GoodEntries = 0; 
    for (int entries=0; entries<Nentries; entries++){
        ch -> GetEntry(entries); 
        if(entries%100000==0){
            cout << "\033[34m\r Processing tree..." << (double)entries/Nentries*100 << "\% done" << flush;
        }//End cout loading


        if(FPMW->DetectorNbr.size()>0){
            GoodEntries += 1;
            for (int DetectorNbr = 0 ; DetectorNbr < FPMW->DetectorNbr.size() ; DetectorNbr ++){
                if (DetectorNbr>=0 && DetectorNbr<4){
                    PositionX[DetectorNbr]->Fill(FPMW->PositionX[DetectorNbr]); 
                    PositionY[DetectorNbr]->Fill(FPMW->PositionY[DetectorNbr]); 
                }


                if (FPMW->PositionX[DetectorNbr]>19 && FPMW->PositionX[DetectorNbr]<20 && DetectorNbr == 0){
                    double XNew = TransfoX(FPMW->PositionX[DetectorNbr]);
                    ZoomDet1->Fill(FPMW->PositionX[DetectorNbr]);
                    ZoomDet1Adjusted->Fill(XNew);
                    //cout << "X :" << FPMW->PositionX[DetectorNbr] << " Xnew: " << XNew << endl;
                }
            } // Loop detnbr
        }//Non empty event

    }//End loop on entries
    cout << endl << "Good entries" << GoodEntries << endl;

    std::vector<TCanvas*> canvas;
    for (int i=0; i<4; i++ ){
        std::string CanvasName = "c" + std::to_string(i) ;
        TCanvas *c = new TCanvas(CanvasName.c_str(),CanvasName.c_str(),960,540);
        c -> Divide(2,1);
        canvas.push_back(c);


        canvas[i] -> cd(1);
        PositionX[i] -> Draw("logY");
        gPad->SetLogy();
        canvas[i] -> cd(2);
        PositionY[i] -> Draw("log");
        gPad->SetLogy();

    }
    TCanvas * CZoom = new TCanvas("CZoom","CZoom",960,540);
    CZoom -> Divide(2,1);
    CZoom -> cd(1);
    ZoomDet1 -> Draw();
    CZoom -> cd(2);
    ZoomDet1Adjusted -> Draw();
    gStyle-> SetOptStat(11111111);



} // end macro

double TransfoX( double X){

    //Tentative proportionelle au NBIN
    double NBin[10] = {5227., 4416.0, 3388.0, 2634.0, 2168.0, 1858.0, 2392.0, 3063.0, 4219.0, 5402.0};
    double NTotBin = 3476.7;
    vector<double> minX , maxX , NewMaxX , NewMinX;
    double offset;
    //Tentative juste a la main
    double a0[10] = {2,1.5,1,1,1,1}; 
    for (int i= 0 ; i<5 ; i++){
        double d = i;
        minX.push_back( 19 + d/10.);
        maxX.push_back( 19 + (d+1.)/10.);
        //cout << "min : " << minX << " max : " << maxX << endl;
        double ScaleMax = 0.1*(0.1 * NBin[i]/NTotBin); 
        double Scale1 =0.1* (X-minX[i]) * (NBin[i]/NTotBin) ;

        //double dilatation2 = abs(X-maxX[i]) * (NBin[i]/NTotBin) ;
        if (i!=0){
            NewMinX.push_back(NewMaxX[i-1]);
            double temp = 1 - exp(-17*abs((ScaleMax * NewMinX[i]+ NewMinX[i]) - 19.5)) ;
            NewMaxX.push_back(ScaleMax * NewMinX[i] *temp + NewMinX[i]);
        }

        else if (i == 0) {
            double temp = 1 - exp(-14*abs((ScaleMax * minX[i]+ minX[i]) - 19.5)) ;
            NewMaxX.push_back(ScaleMax * minX[i] *temp + minX[i]);
            NewMinX.push_back(minX[i]);
        }
        //cout << minX[i] << " " << NewMinX[i] << " " << maxX[i] << " " << NewMaxX[i] << endl;
        if (X>= minX[i] && X< maxX[i]){
            if ( X-int(X)< 0.5){
                X = NewMinX[i] + (X) * Scale1 ;
            }

            else if ( X -int(X) > 0.5 ) {
                //X =  offset - (X) * Scale1 ;
            }
            //X = int(X) + (a0[i]*(X-int(X)));
        }
    }
    return X;
} //end transfo x
