#include <TCanvas.h>
#include <TFile.h>
#include <TGraph.h>
#include <TH2.h>
#include <TSpectrum.h>
#include <vector>

using namespace std;

void PeakFinder(vector<TSpectrum*> *sPeak , vector<int> *vPeak, TH2F *h , vector<vector<Double_t>> *LinePos);
vector<vector<vector<Double_t>>> PeakLinker(vector<vector<Double_t>> *LinePos);

void CutAutoDEE(){

    //retrieve initial histogram
    TFile *inFile = new TFile("Output/DE_E.root","open");
    TH2F *hDE_E = (TH2F*)inFile->Get("DE0234_Ecorr");
    
    //inverse data to use tspectrum
    for (int xBin=1; xBin<= hDE_E->GetNbinsX();xBin++){
        for (int yBin=1; yBin<= hDE_E->GetNbinsY();yBin++){
        hDE_E->SetBinContent(xBin,yBin , -hDE_E->GetBinContent(xBin,yBin));
        }
    }
    //Store result of peak finding
    vector<int> *vPeaks = new vector<int>() ;
    vector<TSpectrum*> *vPeakFinder = new vector<TSpectrum*>();
    vector<vector<Double_t>> *LinePos = new vector<vector<Double_t>> ;

    PeakFinder(vPeakFinder,vPeaks,hDE_E,LinePos);
    vector<vector<vector<Double_t>>> CutLine =  PeakLinker(LinePos);
}

void PeakFinder(vector<TSpectrum*> *sPeak , vector<int> *vPeak, TH2F *h, vector<vector<Double_t>> *LinePos){

    int nbinsX = h->GetNbinsX();
    int step = 10;
    int iter = 0;


    for (int i = 1; i <= nbinsX; i += step) {
        iter +=1;
        int x1 = i;
        int x2 = std::min(i + step - 1, nbinsX);

        TString projName = Form("projY_%d_to_%d", x1, x2);
        TH1D* projY = h->ProjectionY(projName, x1, x2);

        projY->SetTitle(Form("Projection Y: bins %d to %d", x1, x2));

		TSpectrum *PeakFinder = new TSpectrum(55);  
		int nPeaks = PeakFinder->Search(projY,2,"",0.02);
       
        vPeak->push_back(nPeaks);
        sPeak->push_back(PeakFinder);

		if (nPeaks <= 0) {
			cout << "No peaks found!" << endl;
			return;
        }
        
        //Save the position of each peak
        Double_t * x = PeakFinder->GetPositionX();
        
        for(int peaks = 0 ; peaks < nPeaks ; peaks++){
            vector<Double_t> Position(2); 
            Position.at(0) = (double(i) - double(step)/2.0) * h->GetXaxis()->GetBinWidth(4) ; //Retrieve E position
            Position.at(1) = PeakFinder->GetPositionX()[peaks];
            LinePos->push_back(Position);
        }


        //Test
        if (true && iter ==90){
            // Draw the histogram
            TCanvas* c = new TCanvas(Form("c%d",i), Form("Histogram with Peaks%d",i), 800, 600);
            projY->Draw();


        }
    }
}
vector<vector<vector<Double_t>>> PeakLinker(vector<vector<Double_t>> *LinePos){

    vector<vector<vector<Double_t>>> Res ;
    vector<Double_t> X , Y ;

    //Retrieve position of peaks
    for (int ipeak; ipeak<LinePos->size() ; ipeak ++){
        X.push_back(LinePos->at(ipeak).at(0));  
        Y.push_back(LinePos->at(ipeak).at(1));
    }

    // Start from the lowest position in Y 
    // Corresponding to the lowest line in DE-E
    //
    auto minElementY = min_element(Y.begin(),Y.end());
    if (minElementY != Y.end())
    { 
        int indexMin = distance(Y.begin(), minElementY);
        //cout << Y[indexMin] << endl;
        
        //Now we need to find the good threshold between each line in the DE-E
        //To do so we will get the peaks from the same X and get their distance in Y
        //This will be our threshold

        //Create a sub set of data with only same X
        vector<Double_t> YSub; 
        for (int index= 0 ; index < Y.size() ; index ++){
            if (X.at(index) == X.at(indexMin)){
                YSub.push_back(Y.at(index));

            }
        }
        
        for (int index=0 ; index < YSub.size(); index++){        
            cout << " Ysub    " << YSub.at(index) << endl;
        }
        //Sort the vec
        sort(YSub.begin(),YSub.end());

        Double_t Threshold = YSub.at(10) - YSub.at(9) ;
        //cout << YSub.at(0) << "  " << Y.at(indexMin) << endl; //it works
        
        
        
        for (int Point ; Point < Y.size() ; Point ++){
            // We will start with the first point 
            // We need to store all point that are linked together 
        }
    } //end min Y

    return Res;
}
