//#ifdef TICPhysics
#include <TICPhysics.h>
//#endif
#include <TCanvas.h>
#include <TChain.h>
#include <TF1.h>
#include <TFile.h>
#include <TH2.h>
#include <TSpline.h>
#include <fstream>
#include <vector>
#include <TCutG.h>

using namespace std;

TSpline3* MakeSpline(TH2F* hInput, Int_t NCallee, double XMin , double
    XMax, double YMin, double YMax);
void HistoFiller(bool cut=true);
void HistoDrawer(vector<TH2F*> h ,const char* CharName);

//////////////////////////////////////////////////////////////////////////////////////////
void SplineChioAllXY( bool Create = false , bool Cut = false) {


  if (Create == true){
    cout << "Creating SplineInputAll.root" << endl;
    HistoFiller(Cut); //Deactivated after one launch of the macro
    cout << "SplineInputAll.root created and loaded" << endl ;
  }

  TFile* inFile =  TFile::Open("Output/SplineInputAll.root");

  if (inFile && !inFile->IsZombie()) {
    cout << "Histogram loaded" << endl;
  }

  else {
    cerr << "Make your mind" << endl;
  }

  
  int NSegment = 11 ;
  vector<TH2F*> hIC,hICX,hICY;

  for(int seg=0; seg<NSegment; seg++){
    hIC.push_back((TH2F*)inFile->Get(Form("hChio_IC%d_X",seg)));
    hIC.push_back((TH2F*)inFile->Get(Form("hChio_IC%d_Y",seg)));
    hICX.push_back((TH2F*)inFile->Get(Form("hChio_IC%d_X",seg)));
    hICY.push_back((TH2F*)inFile->Get(Form("hChio_IC%d_Y",seg)));
  }

  HistoDrawer(hICX,"CanX");
  HistoDrawer(hICY,"CanY");


  TFile* fspline = new TFile("Output/spline_ICALL_2024.root", "recreate");

  int NHist = static_cast<int>(hIC.size());
  double XMin[NHist] , XMax[NHist], YMin[NHist] , YMax[NHist];
  XMin[0] = -520 ;   XMax[0] = 380 ; YMin[0] = 2000 ; YMax[0] = 9500 ; 
  XMin[1] = -100  ;   XMax[1] = 103  ; YMin[1] = 0 ; YMax[1] = 20000 ; 
  XMin[2] = -520 ;   XMax[2] = 380 ; YMin[2] = 2000 ; YMax[2] = 9500 ; 
  XMin[3] = -100  ;   XMax[3] = 103  ; YMin[3] = 0 ; YMax[3] = 20000 ; 



  for (int i = 0; i < NHist; i++) {
    
    TSpline3 *spline;
    if ( (i%2 == 0) && true) {
      spline = MakeSpline(hIC[i] , i, XMin[2] , XMax[2], YMin[2],
          YMax[2]);
      spline->SetName(Form("fspline_IC%d_X",int((i-2)/2)));   
    }
    else if ( !(i%2 == 0)) {
      spline = MakeSpline(hIC[i] , i, XMin[3] , XMax[3], YMin[3],
          YMax[3]);
      spline->SetName(Form("fspline_IC%d_Y",int((i-2)/2)));   
    }
    spline->Write();
  } //End loop on histogram

  fspline->Close();


} // End spline chio XY

///////////////////////////////////////////////////////////////////////////////////////////
/**
 * This function fill the histograms
 */
void HistoFiller(bool cut) {

  // Input and setters
  TChain* chain = new TChain("PhysicsTree");
  chain->Add("../../../root/analysis/VamosCalib247.root");

  TICPhysics* IC = new TICPhysics() ; 
  double FF_IC_X, FF_IC_Y;

  chain->SetBranchStatus("FF_IC_X", true);
  chain->SetBranchAddress("FF_IC_X", &FF_IC_X);
  chain->SetBranchStatus("FF_IC_Y", true);
  chain->SetBranchAddress("FF_IC_Y", &FF_IC_Y);
  chain->SetBranchStatus("IC", true);
  chain->SetBranchAddress("IC", &IC);

  //spline to correct IC1 as a baseline of IC0
  TFile* fSpline =  TFile::Open("Output/spline_ICALL_2024.root");
  TSpline3* SplineIC1;
  if (fSpline && !fSpline->IsZombie()) {
      SplineIC1 = (TSpline3 *)fSpline->Get(Form("fspline_IC1_Y"));
  }

  //cut
  TFile *fcut = new TFile("Output/CutZ.root","open") ;
  TCutG *CutZ ;
  CutZ = (TCutG*)fcut->Get("CutTry") ;

  // Histograms
  double NSegment = 11;
  vector<TH2F*> hIC1_ICn_X(NSegment), hIC1_ICn_Y(NSegment),  hChio_ICn_Y(NSegment) , hChio_ICn_X(NSegment) ;

  for (int seg = 0 ; seg< NSegment; seg++){
    hIC1_ICn_X.at(seg) = new TH2F(Form("hIC1_IC%d_X",seg),
        Form("hIC1_IC%d_X",seg), 1000, -800, 800, 500, 0, 2);
    hIC1_ICn_Y.at(seg) = new TH2F(Form("hIC1_IC%d_Y",seg),
        Form("hIC1_IC%d_Y",seg), 1000, -160, 160, 500, 0, 2);

    hChio_ICn_Y.at(seg) = new TH2F(Form("hChio_IC%d_Y",seg),
        Form("hChio_IC%d_Y",seg), 1000, -160, 160, 500, 0, 20000);
    hChio_ICn_X.at(seg) = new TH2F(Form("hChio_IC%d_X",seg),
        Form("hChio_IC%d_X",seg), 1000, -800, 800, 500, 0, 20000);

  }


  //===========================================================================================================
  //																	Beginning loop on entries
  //===========================================================================================================

  int Nentries = chain->GetEntries();

  for (int e = 0; e < Nentries; e++) {

    // Printing status
    if (e % 100000 == 0){
      cout <<  "Please wait, we are  " << e * 100. / Nentries << "% done ..\r" << flush;
    }
    chain->GetEntry(e);


    double fIC_raw[11];

    for (int i=0 ; i<11 ; i++){
      fIC_raw[i] = IC->fIC_raw[i]; 
    }


    //===========================================================================================================
    //                                         Cut on DE
    //===========================================================================================================
    //if we want to do cut on DE_E we need to define those parameters
    double DE = 0 ,E = 0, Etot = 0 ;

    for (int seg = 0 ;  seg < sizeof(IC->fIC_raw)/sizeof(IC->fIC_raw[0]) ; seg++ ){
      if (seg >4) E+= IC->fIC_raw[seg] ;
    }

    double IC1corr = (IC->fIC_raw[1]*(1-0.000686068*FF_IC_Y))*(1-4.88238e-05*FF_IC_Y+7.40395e-06*FF_IC_Y*FF_IC_Y);
    double DE_Bis = 0.5*(IC1corr+IC->fIC_raw[2]+IC->fIC_raw[3])+IC->fIC_raw[4];

    Etot = E+DE_Bis;
    //===========================================================================================================
    //                                    Cut Histo
    //===========================================================================================================

    if (cut == true && !(CutZ->IsInside(Etot,DE_Bis)) ){; } // end cut
    else {

      //If the spline to correct IC1 doesnt exist we dont load it
      double IC1_Spline_Corr = fIC_raw[1];
      if (fSpline && !fSpline->IsZombie()) {
        IC1_Spline_Corr = IC->fIC_raw[1] * SplineIC1->Eval(0) / SplineIC1->Eval(FF_IC_Y);
      }
      for(int seg=0; seg<NSegment; seg++){
        hIC1_ICn_X.at(seg)->Fill(FF_IC_X,IC1_Spline_Corr/fIC_raw[seg]);
        hIC1_ICn_Y.at(seg)->Fill(FF_IC_Y,IC1_Spline_Corr/fIC_raw[seg]);
        
        hChio_ICn_X.at(seg)->Fill(FF_IC_X, fIC_raw[seg]);
        hChio_ICn_Y.at(seg)->Fill(FF_IC_Y, fIC_raw[seg]);


      }//loop on segment
    } // end else
  } // end loop on entries

  //===========================================================================================================
  //                                       Output Canvas and files
  //===========================================================================================================

  TFile* outFile = new TFile("Output/SplineInputAll.root", "recreate");
  for(int seg=0; seg<NSegment; seg++){
    hIC1_ICn_X.at(seg)->Write();
    hIC1_ICn_Y.at(seg)->Write();
    hChio_ICn_Y.at(seg)->Write();
    hChio_ICn_X.at(seg)->Write();


  }//loop on segment
  outFile->Close();
} // End HistoFiller

/**
  }
 *  Create the spline for one TH2F
 *
 *  Input :
 *     Pointer to a TH2F
 *
 *
 *  Output:
 *
 *     TSpline3 : the output spline
 *
 */

TSpline3* MakeSpline(TH2F* hInput, Int_t NCallee, double XMin = 0 , double
    XMax = 1000 , double YMin=0 , double YMax = 100) {

  TSpline3* gspline;

  TH1F* hProfile;
  TH1F* pfx; // extended hProfile

  TCanvas* canfit = new TCanvas(Form("CanSpline_%02d",NCallee),
      Form("canfit_%02d",NCallee), 0, 0, 1000, 500);


  //===========================================================================================================
  //                                            Init profile
  //===========================================================================================================

  // Create the TProfile
  hProfile = (TH1F*)hInput->ProfileX();
  hProfile->SetLineWidth(2);
  hProfile->SetDirectory(0);
  canfit->cd();

  hProfile->GetYaxis()->SetRangeUser(YMin, YMax);
  hProfile->Draw();
  //===========================================================================================================
  //                                          First and last bin to get to 6k
  //                                          event get promoted
  //===========================================================================================================
  int FirstBin, LastBin;
  double Treshold = 1;
  for (int bin =1 ; bin<hProfile->GetNbinsX(); bin++) {
    FirstBin = bin;
    if (hProfile->GetBinLowEdge(bin)> XMin) break;
  }
  for (int bin = hProfile->GetNbinsX(); bin>1 ; bin--) {
    LastBin = bin;
    if (hProfile->GetBinLowEdge(bin)< XMax) break;
  }
  cout << FirstBin << " " << LastBin << endl;
  //===========================================================================================================
  //                                          Init Extended profile function
  //===========================================================================================================
  // Create the extended TProfile
  pfx = new TH1F(
      Form("pfx_%02d", Int_t(NCallee/2)), Form("pfx_%02d", Int_t(NCallee/2)), hProfile->GetNbinsX(), hProfile->GetBinLowEdge(1),
      hProfile->GetBinLowEdge(hProfile->GetNbinsX()) + hProfile->GetBinWidth(hProfile->GetNbinsX()));
  pfx->SetLineColor(8);
  pfx->SetDirectory(0);
  //===========================================================================================================
  //                                           Fill extended profile
  //===========================================================================================================

  // fill the extended TProfile
  float newval, lastval, lasterr;
  for (int bin = 1; bin <= FirstBin; bin++) {
    newval = 0;
    pfx->SetBinContent(bin, newval);
  }


  for (int bin = FirstBin; bin <= LastBin; bin++) {
    newval = hProfile->GetBinContent(bin);
    if (newval != 0) {
      pfx->SetBinContent(bin, newval);
      pfx->SetBinError(bin, hProfile->GetBinError(bin));
      lastval = newval;
      lasterr = hProfile->GetBinError(bin);
    }
    else {
      pfx->SetBinContent(bin, lastval);
      pfx->SetBinError(bin, lasterr);
    }
  }
  pfx->Draw("same");

  gspline = new TSpline3(pfx);
  gspline->SetName(Form("fspline_%d", NCallee + 1));

  return gspline;

} // end makespline

void HistoDrawer(vector<TH2F*> h,const char* CharName){

  int NHisto = static_cast<int>(h.size());

  TCanvas* Can= new TCanvas(CharName,CharName,0,0,2000,1000);

  Can->Divide(NHisto);

  for ( int i=0 ; i < NHisto ; i+=1 ){
    //All x are even and all y are odd
    int CanvaNumber = (i) ;
    Can->cd(CanvaNumber+1);
    h[i]->Draw("colz");
  }
}
