//#ifdef TICPhysics
#include <TICPhysics.h>
//#endif
#include <TCanvas.h>
#include <TChain.h>
#include <TF1.h>
#include <TFile.h>
#include <TH2.h>
#include <TSpline.h>
#include <fstream>
#include <vector>
#include <TCutG.h>

using namespace std;

TSpline3* MakeSpline(TH2F* hInput, Int_t NCallee, double XMin , double
    XMax, double YMin, double YMax);
void HistoFiller(bool cut=true);
void HistoDrawer(vector<TH2F*> h ,const char* CharName);

//////////////////////////////////////////////////////////////////////////////////////////
void SplineChioXY( bool Create = false) {

  TFile* inFile =  TFile::Open("Output/Histo.root");

  if (Create == true){
    cout << "Creating Histo.root" << endl;
    HistoFiller(false); //Deactivated after one launch of the macro
    cout << "Histo.root created and loaded" << endl ;
  }


  else if (inFile && !inFile->IsZombie()) {
    cout << "Histogram loaded" << endl;
  }

  else {
    cerr << "Make your mind" << endl;
  }


  vector<TH2F*> hIC,hICX,hICY;
  hIC.push_back((TH2F*)inFile->Get("hChio_IC0_IC1_X_all"));
  hIC.push_back((TH2F*)inFile->Get("hChio_IC0_IC1_Y_all"));
  hIC.push_back((TH2F*)inFile->Get("hChio_IC1_X"));
  hIC.push_back((TH2F*)inFile->Get("hChio_IC1_Y"));
  hIC.push_back((TH2F*)inFile->Get("hChio_IC0_X"));
  hIC.push_back((TH2F*)inFile->Get("hChio_IC0_Y")); 
  hIC.push_back((TH2F*)inFile->Get("hChio_IC0_IC1corr_Y_all"));

  hICX.push_back((TH2F*)inFile->Get("hChio_IC0_IC1_X_all"));
  hICY.push_back((TH2F*)inFile->Get("hChio_IC0_IC1_Y_all"));
  hICX.push_back((TH2F*)inFile->Get("hChio_IC1_X"));
  hICY.push_back((TH2F*)inFile->Get("hChio_IC1_Y"));
  hICX.push_back((TH2F*)inFile->Get("hChio_IC0_X"));
  hICY.push_back((TH2F*)inFile->Get("hChio_IC0_Y")); 
  hICY.push_back((TH2F*)inFile->Get("hChio_IC0_IC1corr_Y_all"));


  HistoDrawer(hICX,"CanX");
  HistoDrawer(hICY,"CanY");


  TFile* fspline = new TFile("Output/spline_Chio_2024.root", "recreate");

  int NHist = static_cast<int>(hIC.size());
  double XMin[NHist] , XMax[NHist], YMin[NHist] , YMax[NHist];
  XMin[0] = -520 ;   XMax[0] = 380 ; YMin[0] = 0 ; YMax[0] = 2 ; 
  XMin[1] = -60  ;   XMax[1] = 63  ; YMin[1] = 0 ; YMax[1] = 2 ; 
  XMin[2] = -520 ;   XMax[2] = 380 ; YMin[2] = 2000 ; YMax[2] = 9500 ; 
  XMin[3] = -60  ;   XMax[3] = 63  ; YMin[3] = 2000 ; YMax[3] = 9500 ; 
  XMin[4] = -520 ;   XMax[4] = 380 ; YMin[4] = 2000 ; YMax[4] = 9500 ; 
  XMin[5] = -60  ;   XMax[5] = 63  ; YMin[5] = 2000 ; YMax[5] = 9500 ; 
  XMin[6] = -60  ;   XMax[6] = 63  ; YMin[6] = 0 ; YMax[6] = 2 ; 



  for (int i = 0; i < NHist; i++) {

    TSpline3* spline = MakeSpline(hIC[i] , i, XMin[i] , XMax[i], YMin[i], YMax[i]);
    if (i < 2) spline->SetName(Form("fsplineIC01_%d", i ));
    else if (i>=2 && i<4) spline->SetName(Form("fsplineIC1_%d",i));   
    else if (i>=4 && i<6) spline->SetName(Form("fsplineIC0_%d",i)); 
    else if (i==6) spline->SetName(Form("fsplineIC0_IC1corr")); 
    spline->Write();
  } //End loop on histogram

  fspline->Close();


} // End spline chio XY

///////////////////////////////////////////////////////////////////////////////////////////
/**
 * This function fill the histograms
 */
void HistoFiller(bool cut) {

  // Input and setters
  TChain* chain = new TChain("PhysicsTree");
  chain->Add("../../../root/analysis/VamosCalib241.root");

  TICPhysics* IC = new TICPhysics() ; 
  double FF_IC_X, FF_IC_Y;

  chain->SetBranchStatus("FF_IC_X", true);
  chain->SetBranchAddress("FF_IC_X", &FF_IC_X);
  chain->SetBranchStatus("FF_IC_Y", true);
  chain->SetBranchAddress("FF_IC_Y", &FF_IC_Y);
  chain->SetBranchStatus("IC", true);
  chain->SetBranchAddress("IC", &IC);

  //cut
  TFile *fcut = new TFile("Output/CutZ.root","open") ;
  TCutG *CutZ ;
  CutZ = (TCutG*)fcut->Get("CutTry") ;

  //spline to correct IC1 as a baseline of IC0
  TFile* fSpline =  TFile::Open("Output/spline_Chio_2024.root");
  TSpline3* SplineIC1 = (TSpline3 *)fSpline->Get(Form("fsplineIC1_3"));

  // Output of the spline

  double fIC0_CorrX;
  double fIC0_CorrY;

  // Histograms

  TH2F* hChio_IC0_IC1_X_all = new TH2F("hChio_IC0_IC1_X_all",
      "hChio_IC0_IC1_X_all", 1000, -800, 800, 500, 1, 1.6);
  TH2F* hChio_IC0_IC1_Y_all = new TH2F("hChio_IC0_IC1_Y_all",
      "hChio_IC0_IC1_Y_all", 1000, -160, 160, 500, 1, 1.6);
  TH2F* hChio_IC1_Y = new TH2F("hChio_IC1_Y",
      "hChio_IC1_Y", 1000, -160, 160, 500, 2000, 9500);
  TH2F* hChio_IC1_X = new TH2F("hChio_IC1_X",
      "hChio_IC1_X", 1000, -800, 800, 500, 2000, 9500);
  TH2F* hChio_IC0_X = new TH2F("hChio_IC0_X",
      "hChio_IC0_X", 1000, -800, 800, 500, 2000, 9500);
  TH2F* hChio_IC0_Y = new TH2F("hChio_IC0_Y",
      "hChio_IC0_Y", 1000, -160, 160, 500, 2000, 9500);

  TH2F* hChio_IC0_IC1corr_Y_all = new TH2F("hChio_IC0_IC1corr_Y_all",
      "hChio_IC0_IC1corr_Y_all", 1000, -160, 160, 500, 0.8, 1.6);

  //===========================================================================================================
  //																	Beginning loop on entries
  //===========================================================================================================

  int Nentries = chain->GetEntries();

  for (int e = 0; e < Nentries; e++) {

    // Printing status
    if (e % 100000 == 0){
      cout <<  "Please wait, we are  " << e * 100. / Nentries << "% done ..\r" << flush;
    }
    chain->GetEntry(e);


    double fIC_raw[11];

    for (int i=0 ; i<11 ; i++){
      fIC_raw[i] = IC->fIC_raw[i]; 
    }

    //if we want to do cut on DE_E we need to define those parameters
    double DE = 0 ,E = 0, Etot = 0 ;

    for (int seg = 0 ;  seg < sizeof(IC->fIC_raw)/sizeof(IC->fIC_raw[0]) ; seg++ ){
     if (seg >4) E+= IC->fIC_raw[seg] ;
    }

    double IC1corr = (IC->fIC_raw[1]*(1-0.000686068*FF_IC_Y))*(1-4.88238e-05*FF_IC_Y+7.40395e-06*FF_IC_Y*FF_IC_Y);
    double DE_Bis = 0.5*(IC1corr+IC->fIC_raw[2]+IC->fIC_raw[3])+IC->fIC_raw[4];

    Etot = E+DE_Bis;


    if (cut == true && !(CutZ->IsInside(Etot,DE_Bis)) ){; } // end cut
    else {

    //if (IC->Eres>11000 && IC->Eres < 12000){
    hChio_IC0_IC1_X_all->Fill(FF_IC_X , fIC_raw[1] / fIC_raw[0] );
    hChio_IC0_IC1_Y_all->Fill(FF_IC_Y , fIC_raw[1] / fIC_raw[0] );

    hChio_IC1_X->Fill(FF_IC_X, fIC_raw[1]);
    hChio_IC1_Y->Fill(FF_IC_Y, fIC_raw[1]);

    hChio_IC0_X->Fill(FF_IC_X, fIC_raw[0]);
    hChio_IC0_Y->Fill(FF_IC_Y, fIC_raw[0]);
    
    double IC1_Spline_Corr = IC->fIC_raw[1] * SplineIC1->Eval(0) / SplineIC1->Eval(FF_IC_Y);
    hChio_IC0_IC1corr_Y_all->Fill(FF_IC_Y , IC1_Spline_Corr / fIC_raw[0] );

    /*
    cout << "*************************" << endl; 
    cout << IC1_Spline_Corr/fIC_raw[0] << endl;
    cout << FF_IC_Y << endl;
    */
    }
  } // end loop on entries

  //===========================================================================================================
  //                                       Output Canvas and files
  //===========================================================================================================

  TFile* outFile = new TFile("Output/Histo.root", "recreate");
  hChio_IC0_IC1_X_all->Write();
  hChio_IC0_IC1_Y_all->Write();
  hChio_IC1_Y->Write();
  hChio_IC1_X->Write();
  hChio_IC0_X->Write();
  hChio_IC0_Y->Write();
  hChio_IC0_IC1corr_Y_all->Write();
  outFile->Close();
} // End HistoFiller

/**
  }
 *  Create the spline for one TH2F
 *
 *  Input :
 *     Pointer to a TH2F
 *
 *
 *  Output:
 *
 *     TSpline3 : the output spline
 *
 */

TSpline3* MakeSpline(TH2F* hInput, Int_t NCallee, double XMin = 0 , double
    XMax = 1000 , double YMin=0 , double YMax = 100) {

  TSpline3* gspline;

  TH1F* hProfile;
  TH1F* pfx; // extended hProfile

  TCanvas* canfit = new TCanvas(Form("CanSpline_%02d",NCallee),
      Form("canfit_%02d",NCallee), 0, 0, 1000, 500);


  //===========================================================================================================
  //                                            Init profile
  //===========================================================================================================

  // Create the TProfile
  hProfile = (TH1F*)hInput->ProfileX();
  hProfile->SetLineWidth(2);
  hProfile->SetDirectory(0);
  canfit->cd();

  hProfile->GetYaxis()->SetRangeUser(YMin, YMax);
  hProfile->Draw();
  //===========================================================================================================
  //                                          First and last bin to get to 6k
  //                                          event get promoted
  //===========================================================================================================
  int FirstBin, LastBin;
  double Treshold = 1;
  for (int bin =1 ; bin<hProfile->GetNbinsX(); bin++) {
    FirstBin = bin;
    if (hProfile->GetBinLowEdge(bin)> XMin) break;
  }
  for (int bin = hProfile->GetNbinsX(); bin>1 ; bin--) {
    LastBin = bin;
    if (hProfile->GetBinLowEdge(bin)< XMax) break;
  }
  cout << FirstBin << " " << LastBin << endl;
  //===========================================================================================================
  //                                          Init Extended profile function
  //===========================================================================================================
  // Create the extended TProfile
  pfx = new TH1F(
      Form("pfx_%02d", NCallee + 1), Form("pfx_%02d", NCallee + 1), hProfile->GetNbinsX(), hProfile->GetBinLowEdge(1),
      hProfile->GetBinLowEdge(hProfile->GetNbinsX()) + hProfile->GetBinWidth(hProfile->GetNbinsX()));
  pfx->SetLineColor(8);
  pfx->SetDirectory(0);

  //===========================================================================================================
  //                                           Fill extended profile
  //===========================================================================================================

  // fill the extended TProfile
  float newval, lastval, lasterr;
  for (int bin = 1; bin <= FirstBin; bin++) {
    newval = 0;
    pfx->SetBinContent(bin, newval);
  }


  for (int bin = FirstBin; bin <= LastBin; bin++) {
    newval = hProfile->GetBinContent(bin);
    if (newval != 0) {
      pfx->SetBinContent(bin, newval);
      pfx->SetBinError(bin, hProfile->GetBinError(bin));
      lastval = newval;
      lasterr = hProfile->GetBinError(bin);
    }
    else {
      pfx->SetBinContent(bin, lastval);
      pfx->SetBinError(bin, lasterr);
    }
  }
  pfx->Draw("same");

  gspline = new TSpline3(pfx);
  gspline->SetName(Form("fspline_%d", NCallee + 1));

  return gspline;

} // end makespline

void HistoDrawer(vector<TH2F*> h,const char* CharName){

  int NHisto = static_cast<int>(h.size());

  TCanvas* Can= new TCanvas(CharName,CharName,0,0,2000,1000);

  Can->Divide(NHisto);

  for ( int i=0 ; i < NHisto ; i+=1 ){
    //All x are even and all y are odd
    int CanvaNumber = (i) ;
    Can->cd(CanvaNumber);
    h[i]->Draw("colz");
  }
}
