const int nb_parameter = 24;
double EBeam = 5.95877*238;
double TargetThickness ;
double* parameter;   
double DeltaE;
double Eres;
double Xcalc;
double Ycalc;
double Zcalc;
double XTarget;
double YTarget;
double Elab = 0;
double ThetaLab = 0;
double Ecal = 0;
int nentries;
int Telescope;
int run_number;

int iteration = 0;
int m_telescope;

int NumericalMinimization(const char* minName = "Minuit2", const char* algoName = "");
double ConstantFactor(const double* parameter);
int LoadData();
NPL::Reaction* elastic;
NPL::EnergyLoss C12C;
NPL::EnergyLoss C12Al;
NPL::EnergyLoss C12Si;
NPL::EnergyLoss U238C;

TH2F* hEx;
TH2F* hExmean_Tel;
TH2F* hExmean_ThetaLab;
TH2F* E_ETh;
TH2F* De_DeTh;
TH1D* hExmean;

TPISTAPhysics* PISTA;
TGraph* C12CQuick = new TGraph("QuickEloss/ELossC.dat");
TGraph* C12CQuickSlow = new TGraph("QuickEloss/ELossCSlow.dat");
TGraph* C12SiQuickDE[8] ;
TGraph* C12SiQuickE[8];
TGraph* C12AlQuick = new TGraph("QuickEloss/ELossAl.dat");
TGraph* C12AlQuickSlow = new TGraph("QuickEloss/ELossAlSlow.dat");

TGraph* gel = new TGraph();
TChain *chain;
///////////////////////////////////////////////////
void Minimization3(){


    /* =================================================================================================================================

       At first we define all the necessary variable

       =================================================================================================================================*/
    for (int i=0; i<8; i++){
        string pathE, pathDE;
        pathE = "QuickEloss/ElossSiE" + to_string(i) + ".dat";
        pathDE =  "QuickEloss/ElossSiDe" + to_string(i) + ".dat";         
        C12SiQuickE[i] = new TGraph(pathE.c_str());
        C12SiQuickDE[i] = new TGraph(pathDE.c_str());
    }
    
    //Energy loss table
    C12C = EnergyLoss("/home/efremovt/nptool/nptool_v3/Projects/pista_e850_main/ana_e850/EnergyLossTable/C12_C.G4table","G4Table",100);
    C12Si = EnergyLoss("/home/efremovt/nptool/nptool_v3/Projects/pista_e850_main/ana_e850/EnergyLossTable/C12_Si.G4table","G4Table",100);
    C12Al = EnergyLoss("/home/efremovt/nptool/nptool_v3/Projects/pista_e850_main/ana_e850/EnergyLossTable/C12_Al.G4table","G4Table",100);
    U238C = EnergyLoss("/home/efremovt/nptool/nptool_v3/Projects/pista_e850_main/ana_e850/EnergyLossTable/U238_C.G4table","G4Table",100);

    //PISTA 
    PISTA = new TPISTAPhysics();
    NPL::InputParser parser("../../../pista_e850_2024.detector");
    PISTA->ReadConfiguration(parser);


    //Define reaction
    elastic   = new NPL::Reaction("238U(12C,12C)238U@1418"); //put here ebeam
    EBeam = U238C.Slow(EBeam,TargetThickness/2,0) ;
    elastic->SetBeamEnergy(EBeam);

    //Define output hist
    hEx = new TH2F("hEx","hEx",6000,0,6000,500,-10,10);
    hExmean_Tel = new TH2F("hExmean_Tel","hExmean_Tel",8,1,9,500,-10,10);
    hExmean_ThetaLab = new TH2F("hExmean_ThetaLab","hExmean_ThetaLab",300,15,75,500,-10,10);
    De_DeTh = new TH2F("De_DeTh","De_DeTh",200,15,50,200,15,50);
    E_ETh = new TH2F("E_ETh","E_ETh",200,35,200,200,35,210);
    TargetThickness=0.44*micrometer;
    //Input tree
    chain = new TChain("tree"); 
    chain->Add("SelectRun/SelectTree_run235.root");

    //Buffer of 21 param corresponding to 3*8 telescope
    //We parametrise the calibration by a 2nd order polynomial
    double buffer[nb_parameter];
    for(int i=0; i<7; i++){
        buffer[3*i] = 0;
        buffer[3*i+1] = 0.0029;
        buffer[3*i+2] = 0.;
    }
    //Last tree buffer are for the xt yt zt
    buffer[nb_parameter - 1 ] = 0 ;
    buffer[nb_parameter - 2 ] = 0 ;
    buffer[nb_parameter - 3 ] = 0 ;


    parameter = new double[nb_parameter];
    parameter = buffer;

    // Here we will import the data
    LoadData();
    ConstantFactor(parameter);
    NumericalMinimization("Minuit2","");

    // Here we print the result
    TCanvas* c1 = new TCanvas("c1","c1",1600,800);
    c1->Divide(2,1);
    c1->cd(1);
    hExmean_Tel->Draw("colz");
    c1->cd(2);
    hExmean_ThetaLab->Draw("colz");



    TCanvas* c2 = new TCanvas("c2","c2",1600,800);
    c2->Divide(2,1);
    c2->cd(1);
    hEx->Draw("colz");

    TCanvas* c3 = new TCanvas("c3","c3",1600,800);
    c3->Divide(2,1);
    c3->cd(1);
    E_ETh->Draw("colz");
    c3->cd(2);
    De_DeTh->Draw("colz");
}

int LoadData(){
    ROOT::EnableImplicitMT();
    chain->SetBranchStatus("DeltaE","true");
    chain->SetBranchAddress("DeltaE",&DeltaE);

    chain->SetBranchStatus("Eres","true");
    chain->SetBranchAddress("Eres",&Eres);

    chain->SetBranchStatus("XTarget","true");
    chain->SetBranchAddress("XTarget",&XTarget);

    chain->SetBranchStatus("YTarget","true");
    chain->SetBranchAddress("YTarget",&YTarget);

    chain->SetBranchStatus("Xcalc","true");
    chain->SetBranchAddress("Xcalc",&Xcalc);

    chain->SetBranchStatus("Ycalc","true");
    chain->SetBranchAddress("Ycalc",&Ycalc);

    chain->SetBranchStatus("Zcalc","true");
    chain->SetBranchAddress("Zcalc",&Zcalc);

    chain->SetBranchStatus("Telescope","true");
    chain->SetBranchAddress("Telescope",&Telescope);

    chain->SetBranchStatus("run_number","true");
    chain->SetBranchAddress("run_number",&run_number);

    chain->SetBranchAddress("PISTA",&PISTA);

    gel = elastic->GetKinematicLine3();

    nentries = chain->GetEntries();

    return 0;
}

double ConstantFactor(const double*parameter){
    double distance=100;
    iteration ++ ;
    hExmean_Tel->Reset();
    hExmean_ThetaLab->Reset();
    for(int i=0; i<nentries; i++){
        chain->GetEntry(i);

        //This is because telescope 7 is down
        if(Telescope==8) {Telescope=7;}
        int indice = 3*(Telescope-1); //just start index at 0

        Ecal = parameter[indice] + parameter[indice+1]*Eres + parameter[indice+2]*Eres*Eres;



        if(Xcalc!=-1000 && Ycalc!=-1000 && Zcalc!=-1000){
            TVector3 PositionOnTarget = TVector3(XTarget+parameter[24-3],YTarget+parameter[25-3],parameter[26-3]);
            TVector3 HitPosition = TVector3(Xcalc,Ycalc,Zcalc);
            TVector3 BeamDirection = TVector3(0,0,1);
            TVector3 HitDirection = HitPosition - PositionOnTarget;
            ThetaLab = HitDirection.Angle(BeamDirection);

            Elab = C12AlQuick->Eval(Ecal); // 1 deadlayer
            Elab = C12AlQuick->Eval(Elab); // 2 deadlayer
            Elab += DeltaE;
            Elab = C12AlQuick->Eval(Elab); // 3 deadlayer
            Elab = C12CQuick->Eval(Elab);

            double ElabTh = gel->Eval(ThetaLab/deg);
            double ThckDeadLayer = 0.5*micrometer;
            double AnglePista ;
            double C12E, DETh, ETh;

            TVector3 NormaliPista = PISTA->GetDetectorNormal(0);
            AnglePista = (HitDirection.Angle(NormaliPista))    ;

            //Analysis for theoretical value
            C12E = ElabTh; 

            C12E = C12CQuickSlow->Eval(C12E);  //Lost in the target
            C12E = C12AlQuickSlow->Eval(C12E);      //1st dead layer
            DETh =  C12E - C12SiQuickDE[Telescope-1]->Eval(C12E);
            C12E -= DETh;
            C12E = C12AlQuickSlow->Eval(C12E);      // 2 other dead layer
            C12E = C12AlQuickSlow->Eval(C12E);      // 3 other dead layer
            ETh =  C12E - C12SiQuickE[Telescope-1]->Eval(C12E);

            // cout  << ETh << "  " << DETh << endl;
            //cout << Elab << " " << Elab_th << endl; 

            //Fill Histo

            E_ETh->Fill(Elab- DeltaE,ETh);
            De_DeTh->Fill(DeltaE,DETh);
            double Ex=-1000;

            Ex = elastic->ReconstructRelativistic(Elab,ThetaLab);
            if(abs(Ex)<10){
                hEx->Fill(iteration,Ex);
                hExmean_Tel->Fill(indice/3+1,Ex);
                hExmean_ThetaLab->Fill(ThetaLab*180/3.1416,Ex);
            }

        }
    }

    double Emean[7];
    double sigma[7];
    double AvEmean=0;  
    for(int i=0;i<7;i++)
    {
        // C12 case
        hExmean = hExmean_Tel->ProjectionY(Form("hExmean_%d",i+1),i+1,i+1);
        Emean[i] = hExmean->GetMean(); // get mean of gaus instead
        double Emin = Emean[i] - 6;
        double Emax = Emean[i] + 6;
        hExmean->GetXaxis()->SetRangeUser(Emin,Emax);
        Emean[i] = hExmean->GetMean();
        sigma[i] = hExmean->GetRMS();
        distance += pow(Emean[i],2) + pow(sigma[i]-0.4,2);
        AvEmean += Emean[i];

    }
    if(iteration%1==0){
        cout << "Number of iterations: " << iteration  << endl;
        cout << "Emean= " << AvEmean << endl;
        for(int j=0;j<7;j++){
            cout << "Telescope " << j << " : ";
            cout << "  "<< parameter[j*3] << "  " << parameter[j*3+1] << "  " << parameter[j*3+2] <<  endl;
        }
        cout << "Xt Yt Zt :      " << parameter[nb_parameter-3] << "  " << parameter[nb_parameter-2] << "  " << parameter[nb_parameter-1] << endl;
        cout << "distance= " << distance << endl;
    }
    return distance;
}



int NumericalMinimization(const char * minName = "Minuit2",
        const char *algoName = "")
{
    // create minimizer giving a name and a name (optionally) for the specific
    // algorithm
    // possible choices are:
    //     minName                  algoName
    // Minuit /Minuit2             Migrad, Simplex,Combined,Scan  (default is Migrad)
    //  Minuit2                     Fumili2
    //  Fumili
    //  GSLMultiMin                ConjugateFR, ConjugatePR, BFGS,
    //                              BFGS2, SteepestDescent
    //  GSLMultiFit
    //   GSLSimAn
    //   Genetic
    ROOT::Math::Minimizer* minimum = ROOT::Math::Factory::CreateMinimizer(minName, algoName);
    if (!minimum) { std::cerr << "Error: cannot create minimizer \"" << minName
        << "\". Maybe the required library was not built?" << std::endl;
        return 1;
    }

    // set tolerance , etc...
    minimum->SetMaxFunctionCalls(1000000); // for Minuit/Minuit2
    minimum->SetMaxIterations(10000);  // for GSL
    minimum->SetTolerance(0.001);
    minimum->SetPrecision(0.0001);
    minimum->SetPrintLevel(1);

    // create function wrapper for minimizer
    // a IMultiGenFunction type
    ROOT::Math::Functor f(&ConstantFactor,nb_parameter);
    for(int i=0; i<7; i++){

        int indice = 3*i;
        if(i==6) i=7;
        string par0_name = "p0_E" + to_string(i+1);
        string par1_name = "p1_E" + to_string(i+1);
        string par2_name = "p2_E" + to_string(i+1);

        minimum->SetLimitedVariable(indice  ,par0_name,0     ,0.01  ,-0.2      ,0.2        );
        minimum->SetLimitedVariable(indice+1,par1_name,0.0029,0.0001,0.9*0.0029,1.1*0.0029 );
        minimum->SetLimitedVariable(indice+2,par2_name,0     ,0.0001,-1.e-8    ,1.e-8      );
    }
    minimum->SetLimitedVariable(24-3,"Xt",0,0.01,-2,2);
    minimum->SetLimitedVariable(25-3,"Yt",0,0.01,-2,2);
    minimum->SetLimitedVariable(26-3,"Zt",0,0.01,-1,1);
    minimum->SetFunction(f);


    // do the minimization
    minimum->Minimize();

    const double *xs = minimum->X();
    std::cout << "Minimum: f(" << xs[0] << "," << xs[1] << "): "
        << minimum->MinValue()  << std::endl;


    ofstream ofile;
    string filename = "PISTA_BACK_E_min.cal";
    ofile.open(filename.c_str());

    for(int i=0; i<7; i++){
        int indice = 3*i;
        if(i==6) i=7;
        string token = "PISTA_T" + to_string(i+1) + "_BACK_E";

        ofile << token << " " << xs[indice] << " " << xs[indice+1] << " " << xs[indice+2] << endl;
    }
    ofile.close();

    ofstream ofile2;
    string filename2 = "AnalysisConfig_min.dat";
    ofile2.open(filename2.c_str());
    ofile2 <<  "AnalysisConfig" << endl;
    ofile2 <<  " XTARGET_OFFSET " << xs[24-3] << endl;
    ofile2 <<  " YTARGET_OFFSET " << xs[25-3] << endl;
    ofile2 <<  " ZTARGET_OFFSET " << xs[26-3] << endl;
    //ofile2 <<  " BEAM_THETAX " << xs[27-3] << endl;
    //ofile2 <<  " BEAM_THETAY " << xs[28-3] << endl;
    ofile2.close();

    cout << "MinValue = " << minimum->MinValue() << endl;
    ConstantFactor(xs);

    // expected minimum is 0
    if ( minimum->MinValue()  < 1.E-4 )
        std::cout << "Minimizer " << minName << " - " << algoName
            << "   converged to the right minimum" << std::endl;
    else {
        std::cout << "Minimizer " << minName << " - " << algoName
            << "   failed to converge !!!" << std::endl;
        Error("NumericalMinimization","fail to converge");
    }
    return 0;
}


