#include "convert.h"
///////////////////////////////////////////////////
void convert(int run=204){
  m_pista = new TPISTAData();
  m_fpmw = new TFPMWData();
  m_ic = new TICData();
  m_time = new TTimeData();
  m_exogam = new TExogamCalData();

  int Year = 2024;
  // input tree //
  input_tree = new TChain("AD");
  input_tree->Add(Form("/home/efremovt/Data/2to_SSD/%i/%i_RootA/r0%i*.root",Year,Year,run));
  InitInputTree();


  // output tree //
  TString output_path;
  output_path = Form("~/Data/2to_SSD/%i/np_raw/",Year);
  TString filename = output_path+Form("run_raw_%i.root",run);
  TFile* ofile = TFile::Open(filename,"recreate");
  TTree* output_tree = new TTree("RawTree","RawTree");
  //InitOutputTree();
  output_tree->Branch("PISTA","TPISTAData",&m_pista);
  output_tree->Branch("FPMW","TFPMWData",&m_fpmw);
  output_tree->Branch("IC","TICData",&m_ic);
  output_tree->Branch("Time","TTimeData",&m_time);
  output_tree->Branch("Exogam","TExogamCalData",&m_exogam);

  output_tree->Branch("T_TMW0_FPMW0",&T_TMW0_FPMW0,"T_TMW0_FPMW0/F");
  output_tree->Branch("T_TMW0_FPMW1",&T_TMW0_FPMW1,"T_TMW0_FPMW1/F");
  output_tree->Branch("T_TMW1_FPMW0",&T_TMW1_FPMW0,"T_TMW1_FPMW0/F");
  output_tree->Branch("T_TMW1_FPMW1",&T_TMW1_FPMW1,"T_TMW1_FPMW1/F");
  output_tree->Branch("fVAMOS_TS_sec",&fVAMOS_TS_sec);
  output_tree->Branch("fPISTA_TS_sec",&fPISTA_TS_sec);
  //4 diff times
  
  output_tree->Branch("MTOF_FP0_T0VN",MTOF_FP0_T0VN);
  output_tree->Branch("MTOF_FP0_T1VN",MTOF_FP0_T1VN);
  output_tree->Branch("MTOF_FP1_T0VN",MTOF_FP1_T0VN);
  output_tree->Branch("MTOF_FP1_T1VN",MTOF_FP1_T1VN);

  output_tree->Branch("MTOF_FP0_T0VM",&MTOF_FP0_T0VM);
  output_tree->Branch("MTOF_FP0_T1VM",&MTOF_FP0_T1VM);
  output_tree->Branch("MTOF_FP1_T0VM",&MTOF_FP1_T0VM);
  output_tree->Branch("MTOF_FP1_T1VM",&MTOF_FP1_T1VM);

  output_tree->Branch("MTOF_FP0_T0V",MTOF_FP0_T0V);
  output_tree->Branch("MTOF_FP0_T1V",MTOF_FP0_T1V);
  output_tree->Branch("MTOF_FP1_T0V",MTOF_FP1_T0V);
  output_tree->Branch("MTOF_FP1_T1V",MTOF_FP1_T1V);






  auto start = std::chrono::high_resolution_clock::now();
  int nentries = input_tree->GetEntries();
  for(int i=0; i<nentries; i++){
    Clear();
    input_tree->GetEntry(i);


    if (i % 100000  == 0 && i > 0 ) {
            auto now = std::chrono::high_resolution_clock::now();
            std::chrono::duration<double> elapsed = now - start;
            double avgTimePerIteration = elapsed.count() / i;
            double timeLeft = avgTimePerIteration * (nentries - i);

            std::cout << "********** Estimated time left: " << int(timeLeft) << " seconds **********" << "\r" << flush;
        }
    
   fVAMOS_TS_sec = TMWPat_0TS*1e-8;
   fPISTA_TS_sec = PISTA_TS*1e-8;

    // IC //
    for (int i=0; i<ICRawM; i++){
      m_ic->SetIC_TS(ICRawTS[i]);
      m_ic->SetIC_Section(i+1);
      m_ic->SetIC_Charge(IC[i]);
    }

    // Time //
    
    for (int i=0; i<MTOF_FP0_T0VM;i++){
        m_time->SetTS_MWPC13(MTOF_FP0_T0VTS[i]); 
        m_time->SetTime_MWPC13(MTOF_FP0_T0V[i]); 
        m_time->SetSection_MWPC3(MTOF_FP0_T0VN[i]); 
    }
    for (int i=0; i<MTOF_FP0_T1VM;i++){
        m_time->SetTS_MWPC23(MTOF_FP0_T1VTS[i]);
        m_time->SetTime_MWPC23(MTOF_FP0_T1V[i]);
    }

    for (int i=0; i<MTOF_FP1_T0VM;i++){
        m_time->SetTS_MWPC14(MTOF_FP1_T0VTS[i]); 
        m_time->SetTime_MWPC14(MTOF_FP1_T0V[i]); 
        m_time->SetSection_MWPC4(MTOF_FP1_T0VN[i]); 
    }

    for (int i=0; i<MTOF_FP1_T1VM;i++){
        m_time->SetTS_MWPC24(MTOF_FP1_T1VTS[i]); 
        m_time->SetTime_MWPC24(MTOF_FP1_T1V[i]); 
    }



    // -TMW1- //
    for(int p=0;p<TMW1_XRawM; p++){
        int strip = TMW1_XRawNr[p];
        double charge = TMW1_XRaw[p];
        m_fpmw->SetFPMW_X(1,strip,charge);
    }
    for(int p=0;p<TMW1_YRawM; p++){
        int strip = TMW1_YRawNr[p];
        double charge = TMW1_YRaw[p];
        m_fpmw->SetFPMW_Y(1,strip,charge);
    }
    // -TMW2- //
    for(int p=0;p<TMW2_XRawM; p++){
        int strip = TMW2_XRawNr[p];
        double charge = TMW2_XRaw[p];
        m_fpmw->SetFPMW_X(2,strip,charge);
    }
    for(int p=0;p<TMW2_YRawM; p++){
        int strip = TMW2_YRawNr[p];
        double charge = TMW2_YRaw[p];
        m_fpmw->SetFPMW_Y(2,strip,charge);
    }

    // -FPMW0- //
    for(int p=0;p<FPMW0_XRawM; p++){
        int strip = FPMW0_XRawNr[p];
        double charge = FPMW0_XRaw[p];
        m_fpmw->SetFPMW_X(3,strip,charge);
    }
    for(int p=0;p<FPMW0_YRawM; p++){
        int strip = FPMW0_YRawNr[p];
        double charge = FPMW0_YRaw[p];
        m_fpmw->SetFPMW_Y(3,strip,charge);
    }
    // -FPMW1- //
    for(int p=0;p<FPMW1_XRawM; p++){
        int strip = FPMW1_XRawNr[p];
        double charge = FPMW1_XRaw[p];
        m_fpmw->SetFPMW_X(4,strip,charge);
    }
    for(int p=0;p<FPMW1_YRawM; p++){
        int strip = FPMW1_YRawNr[p];
        double charge = FPMW1_YRaw[p];
        m_fpmw->SetFPMW_Y(4,strip,charge);
    }

    if(fVAMOS_TS_sec>0 || fPISTA_TS_sec>0){
        if(OutersVM<25){
            for(int p=0; p<Inner6MVM; p++){
                //if(Inner6MVM==2) cout << "********p= " << p << endl;
                int crystal = Inner6MVN[p];
                double E = Inner6MV[p];
                double EHG = Inner20MV[p];
                double TDC = DeltaTV[p];  // Existe pas en 2024
                unsigned long long TS = Inner6MVTS[p];
                int seg = -1;
                double outers1 = -1000;
                double outers2 = -1000;
                double outers3 = -1000;
                double outers4 = -1000;

                //cout << "*****************mult= " << OutersVM << endl;
                for(int i=0; i<OutersVM; i++){
                    seg = OutersVN[i] - 4*crystal;
                    // cout << i << " " << OutersVN[i] << " " << crystal << " " << seg << endl;
                    if(seg==0) outers1 = OutersV[i];
                    else if(seg==1) outers2 = OutersV[i];
                    else if(seg==2) outers3 = OutersV[i];
                    else if(seg==3) outers4 = OutersV[i];
                }
                m_exogam->SetExo(crystal,E,EHG,TS,TDC,0,0,outers1,outers2,outers3,outers4);
                //cout << outers1 << " " << outers2 << " " << outers3 << " " << outers4 << endl;
                //if(Inner6MVM==2){
                //  cout << "setting p= " << p << " " << outers1 << " " << outers2 << " " << outers3 << " " << outers4 << endl;
                //}
            }
        }
    }

    if(fPISTA_TS_sec>0 || fVAMOS_TS_sec>0)
        output_tree->Fill();
  }

  output_tree->Write();
  ofile->Close();


}

///////////////////////////////////////////////////
void Clear(){
    m_fpmw->Clear();
    m_ic->Clear();
    m_time->Clear();
    m_pista->Clear();
    m_exogam->Clear();
}


///////////////////////////////////////////////////
void InitInputTree(){
    input_tree->SetBranchAddress("PISTA",&m_pista);

    // TAC
    input_tree->SetBranchStatus("T_TMW1_FPMW0_C","true");
    input_tree->SetBranchAddress("T_TMW1_FPMW0_C",&T_TMW0_FPMW0);
    input_tree->SetBranchStatus("T_TMW1_FPMW1_C","true");
    input_tree->SetBranchAddress("T_TMW1_FPMW1_C",&T_TMW0_FPMW1);
    input_tree->SetBranchStatus("T_TMW2_FPMW0_C","true");
    input_tree->SetBranchAddress("T_TMW2_FPMW0_C",&T_TMW1_FPMW0);
    input_tree->SetBranchStatus("T_TMW2_FPMW1_C","true");
    input_tree->SetBranchAddress("T_TMW2_FPMW1_C",&T_TMW1_FPMW1);

    // Pat
    //input_tree->SetBranchStatus("MTOF_FP0_T0VN","true");
    //input_tree->SetBranchAddress("MTOF_FP0_T0VN",&fFPMWPat_0RawNr);
    input_tree->SetBranchStatus("TMWPat_00TS","true");
    input_tree->SetBranchAddress("TMWPat_00TS",&TMWPat_0TS);
    input_tree->SetBranchStatus("PISTA_TS","true");
    input_tree->SetBranchAddress("PISTA_TS",&PISTA_TS);

    //MTOF

    input_tree->SetBranchStatus("MTOF_FP0_T0VN","true");
    input_tree->SetBranchAddress("MTOF_FP0_T0VN",&MTOF_FP0_T0VN);
    input_tree->SetBranchStatus("MTOF_FP1_T0VN","true");
    input_tree->SetBranchAddress("MTOF_FP1_T0VN",&MTOF_FP1_T0VN);
    input_tree->SetBranchStatus("MTOF_FP0_T1VN","true");
    input_tree->SetBranchAddress("MTOF_FP0_T1VN",&MTOF_FP0_T1VN);
    input_tree->SetBranchStatus("MTOF_FP1_T1VN","true");
    input_tree->SetBranchAddress("MTOF_FP1_T1VN",&MTOF_FP1_T1VN);

    input_tree->SetBranchStatus("MTOF_FP0_T0VM","true");
    input_tree->SetBranchAddress("MTOF_FP0_T0VM",&MTOF_FP0_T0VM);
    input_tree->SetBranchStatus("MTOF_FP1_T0VM","true");
    input_tree->SetBranchAddress("MTOF_FP1_T0VM",&MTOF_FP1_T0VM);
    input_tree->SetBranchStatus("MTOF_FP0_T1VM","true");
    input_tree->SetBranchAddress("MTOF_FP0_T1VM",&MTOF_FP0_T1VM);
    input_tree->SetBranchStatus("MTOF_FP1_T1VM","true");
    input_tree->SetBranchAddress("MTOF_FP1_T1VM",&MTOF_FP1_T1VM);

    input_tree->SetBranchStatus("MTOF_FP0_T0V","true");
    input_tree->SetBranchAddress("MTOF_FP0_T0V",&MTOF_FP0_T0V);
    input_tree->SetBranchStatus("MTOF_FP1_T0V","true");
    input_tree->SetBranchAddress("MTOF_FP1_T0V",&MTOF_FP1_T0V);
    input_tree->SetBranchStatus("MTOF_FP0_T1V","true");
    input_tree->SetBranchAddress("MTOF_FP0_T1V",&MTOF_FP0_T1V);
    input_tree->SetBranchStatus("MTOF_FP1_T1V","true");
    input_tree->SetBranchAddress("MTOF_FP1_T1V",&MTOF_FP1_T1V);


    input_tree->SetBranchStatus("MTOF_FP0_T0VTS","true");
    input_tree->SetBranchAddress("MTOF_FP0_T0VTS",&MTOF_FP0_T0VTS);
    input_tree->SetBranchStatus("MTOF_FP1_T0VTS","true");
    input_tree->SetBranchAddress("MTOF_FP1_T0VTS",&MTOF_FP1_T0VTS);
    input_tree->SetBranchStatus("MTOF_FP0_T1VTS","true");
    input_tree->SetBranchAddress("MTOF_FP0_T1VTS",&MTOF_FP0_T1VTS);
    input_tree->SetBranchStatus("MTOF_FP1_T1VTS","true");
    input_tree->SetBranchAddress("MTOF_FP1_T1VTS",&MTOF_FP1_T1VTS);

    // IC
    input_tree->SetBranchStatus("IC","true");
    input_tree->SetBranchAddress("IC",&IC);

    input_tree->SetBranchStatus("ICRawTS","true");
    input_tree->SetBranchAddress("ICRawTS",&ICRawTS);

    input_tree->SetBranchStatus("ICRawM","true");
    input_tree->SetBranchAddress("ICRawM",&ICRawM);

    // FPMW0-X
    input_tree->SetBranchStatus("FPMW0_XRawM","true");
    input_tree->SetBranchAddress("FPMW0_XRawM",&FPMW0_XRawM);
    input_tree->SetBranchStatus("FPMW0_XRaw","true");
    input_tree->SetBranchAddress("FPMW0_XRaw",FPMW0_XRaw);
    input_tree->SetBranchStatus("FPMW0_XRawNr","true");
    input_tree->SetBranchAddress("FPMW0_XRawNr",FPMW0_XRawNr);
    // FPMW0-Y
    input_tree->SetBranchStatus("FPMW0_YRawM","true");
    input_tree->SetBranchAddress("FPMW0_YRawM",&FPMW0_YRawM);
    input_tree->SetBranchStatus("FPMW0_YRaw","true");
    input_tree->SetBranchAddress("FPMW0_YRaw",FPMW0_YRaw);
    input_tree->SetBranchStatus("FPMW0_YRawNr","true");
    input_tree->SetBranchAddress("FPMW0_YRawNr",FPMW0_YRawNr);
    // FPMW1-X
    input_tree->SetBranchStatus("FPMW1_XRawM","true");
    input_tree->SetBranchAddress("FPMW1_XRawM",&FPMW1_XRawM);
    input_tree->SetBranchStatus("FPMW1_XRaw","true");
    input_tree->SetBranchAddress("FPMW1_XRaw",FPMW1_XRaw);
    input_tree->SetBranchStatus("FPMW1_XRawNr","true");
    input_tree->SetBranchAddress("FPMW1_XRawNr",FPMW1_XRawNr);
    // FPMW0-Y
    input_tree->SetBranchStatus("FPMW1_YRawM","true");
    input_tree->SetBranchAddress("FPMW1_YRawM",&FPMW1_YRawM);
    input_tree->SetBranchStatus("FPMW1_YRaw","true");
    input_tree->SetBranchAddress("FPMW1_YRaw",FPMW1_YRaw);
    input_tree->SetBranchStatus("FPMW1_YRawNr","true");
    input_tree->SetBranchAddress("FPMW1_YRawNr",FPMW1_YRawNr);
    // TMW1-X
    input_tree->SetBranchStatus("TMW1_XRawM","true");
    input_tree->SetBranchAddress("TMW1_XRawM",&TMW1_XRawM);
    input_tree->SetBranchStatus("TMW1_XRaw","true");
    input_tree->SetBranchAddress("TMW1_XRaw",TMW1_XRaw);
    input_tree->SetBranchStatus("TMW1_XRawNr","true");
    input_tree->SetBranchAddress("TMW1_XRawNr",TMW1_XRawNr);
    // TMW1-Y
    input_tree->SetBranchStatus("TMW1_YRawM","true");
    input_tree->SetBranchAddress("TMW1_YRawM",&TMW1_YRawM);
    input_tree->SetBranchStatus("TMW1_YRaw","true");
    input_tree->SetBranchAddress("TMW1_YRaw",TMW1_YRaw);
    input_tree->SetBranchStatus("TMW1_YRawNr","true");
    input_tree->SetBranchAddress("TMW1_YRawNr",TMW1_YRawNr);
    // TMW2-X
    input_tree->SetBranchStatus("TMW2_XRawM","true");
    input_tree->SetBranchAddress("TMW2_XRawM",&TMW2_XRawM);
    input_tree->SetBranchStatus("TMW2_XRaw","true");
    input_tree->SetBranchAddress("TMW2_XRaw",TMW2_XRaw);
    input_tree->SetBranchStatus("TMW2_XRawNr","true");
    input_tree->SetBranchAddress("TMW2_XRawNr",TMW2_XRawNr);
    // TMW2-Y
    input_tree->SetBranchStatus("TMW2_YRawM","true");
    input_tree->SetBranchAddress("TMW2_YRawM",&TMW2_YRawM);
    input_tree->SetBranchStatus("TMW2_YRaw","true");
    input_tree->SetBranchAddress("TMW2_YRaw",TMW2_YRaw);
    input_tree->SetBranchStatus("TMW2_YRawNr","true");
    input_tree->SetBranchAddress("TMW2_YRawNr",TMW2_YRawNr);
    // Exogam
    input_tree->SetBranchStatus("Inner6MVM","true");
    input_tree->SetBranchAddress("Inner6MVM",&Inner6MVM);
    input_tree->SetBranchStatus("Inner6MV","true");
    input_tree->SetBranchAddress("Inner6MV",Inner6MV);
    input_tree->SetBranchStatus("Inner6MVN","true");
    input_tree->SetBranchAddress("Inner6MVN",Inner6MVN);
    input_tree->SetBranchStatus("Inner6MVTS","true");
    input_tree->SetBranchAddress("Inner6MVTS",Inner6MVTS);

    input_tree->SetBranchStatus("OutersVM","true");
    input_tree->SetBranchAddress("OutersVM",&OutersVM);
    input_tree->SetBranchStatus("OutersV","true");
    input_tree->SetBranchAddress("OutersV",OutersV);
    input_tree->SetBranchStatus("OutersVN","true");
    input_tree->SetBranchAddress("OutersVN",OutersVN);

    input_tree->SetBranchStatus("Inner20MV","true");
    input_tree->SetBranchAddress("Inner20MV",Inner20MV);

}

///////////////////////////////////////////////////
//void InitOutputTree(){
//  output_tree->Branch("PISTA","TPISTAData",&m_pista);
//  output_tree->Branch("FPMW","TFPMWData",&m_fpmw);
//  output_tree->Branch("IC","TICData",&m_ic);
//
//  output_tree->Branch("T_TMW0_FPMW0",&T_TMW0_FPMW0,"T_TMW0_FPMW0/F");
//  output_tree->Branch("T_TMW0_FPMW1",&T_TMW0_FPMW1,"T_TMW0_FPMW1/F");
//  output_tree->Branch("T_TMW1_FPMW0",&T_TMW1_FPMW0,"T_TMW1_FPMW0/F");
//  output_tree->Branch("T_TMW1_FPMW1",&T_TMW1_FPMW1,"T_TMW1_FPMW1/F");
//}


