/*****************************************************************************
 * Copyright (C) 2009-2014    this file is part of the NPTool Project        *
 *                                                                           *
 * For the licensing terms see $NPTOOL/Licence/NPTool_Licence                *
 * For the list of contributors see $NPTOOL/Licence/Contributors             *
 *****************************************************************************/

/*****************************************************************************
 * Original Author: Adrien MATTA  contact address: a.matta@surrey.ac.uk      *
 *                                                                           *
 * Creation Date  : march 2025                                               *
 * Last update    :                                                          *
 *---------------------------------------------------------------------------*
 * Decription:                                                               *
 * Class describing the property of an Analysis object                       *
 *                                                                           *
 *---------------------------------------------------------------------------*
 * Comment:                                                                  *
 *                                                                           *
 *                                                                           *
 *****************************************************************************/
#include<iostream>
using namespace std;
#include"Analysis.h"
#include"NPAnalysisFactory.h"
#include"NPDetectorManager.h"
#include"NPOptionManager.h"
#include"RootOutput.h"
#include"RootInput.h"
////////////////////////////////////////////////////////////////////////////////
Analysis::Analysis(){
}
////////////////////////////////////////////////////////////////////////////////
Analysis::~Analysis(){
}

////////////////////////////////////////////////////////////////////////////////
void Analysis::Init(){
  m_ChiNu= (TChiNuPhysics*) m_DetectorManager->GetDetector("ChiNu");
  InitialConditions = new TInitialConditions();
  InteractionCoordinates = new TInteractionCoordinates();
  ReactionConditions = new TReactionConditions();

  InitInputBranch();
  InitOutputBranch();

  my_Reaction = new NPL::Reaction("1n(238U,1n)238U@0.75");

  neutron = new NPL::Nucleus("1n");
}

////////////////////////////////////////////////////////////////////////////////
void Analysis::TreatEvent(){
  ReInitValue();
  Einit = InitialConditions->GetKineticEnergy(0);
  double init_ThetaLab   = ReactionConditions->GetTheta(0)*deg;
  double init_BeamEnergy = ReactionConditions->GetBeamEnergy();
  //neutron->SetKineticEnergy(init_BeamEnergy);
  neutron->SetKineticEnergy(Einit);
  double beam_TOF = neutron->GetTimeOfFlight();

  double Xtarget = InitialConditions->GetIncidentPositionX();
  double Ytarget = InitialConditions->GetIncidentPositionY();
  double Ztarget = 0;//InitialConditions->GetIncidentPositionZ();
  TVector3 TargetPos = TVector3(Xtarget,Ytarget,Ztarget);
  //  cout << "Xtarget=" << Xtarget << endl;

  unsigned int IntCoord_mult=InteractionCoordinates->GetDetectedMultiplicity();
  InCellHitMultiplicity=IntCoord_mult;
  
  // cout << "m_ChiNu->Energy.size()=" << m_ChiNu->Energy.size() << endl;
  //  cout << "IntCoord_mult=" << IntCoord_mult << endl;
  for(int i=0; i<m_ChiNu->Energy.size(); i++){
    double Rdet, R;

    TVector3 DetPos = m_ChiNu->GetVectorDetectorPosition(m_ChiNu->DetectorNumber[i]);
    TVector3 HitPos = DetPos-TargetPos;
    Rdet = DetPos.Mag() ;
    R    = Rdet*mm;
    
    Distance.push_back(R);	
    Det.push_back(m_ChiNu->DetectorNumber[i]); 
    T.push_back(m_ChiNu->Time[i]);
    double T_stop = (m_ChiNu->Time[i]); 
    neutron->SetBeta(  (Rdet/T_stop) / c_light); 
    E.push_back(m_ChiNu->Energy[i]);
    Elab.push_back(neutron->GetEnergy());


    double DeltaTheta      = atan(89.0/Rdet);
    double exp_ThetaLab    = m_ChiNu->GetVectorDetectorPosition(m_ChiNu->DetectorNumber[i]).Theta();
    double random_ThetaLab = ra.Uniform(exp_ThetaLab-DeltaTheta, exp_ThetaLab+DeltaTheta);
    double dEx             = my_Reaction->ReconstructRelativistic(Elab[i], random_ThetaLab);
      
    ThetaLab.push_back(random_ThetaLab/deg);
    //ThetaLab.push_back(exp_ThetaLab/deg);
    Ex.push_back(dEx);
  }

  // Treatment of different hit in one detector due to neutron scattering
  for(int i=0; i<m_ChiNu->Energy.size(); i++){
    for(unsigned int j=0; j<IntCoord_mult; j++)
      {// Associates with each recorded hit the full scattering pattern of the simulated neutron.
       // At a later step, the position of each scattering point is compared with the size of the
       // detector to determine if the hit was inside the detector or not.
	double Rdet, R;

	TVector3 DetPos = m_ChiNu->GetVectorDetectorPosition(m_ChiNu->DetectorNumber[i]);
	TVector3 HitPos = DetPos-TargetPos;

	Rdet = DetPos.Mag() ;
	R= Rdet*mm;
	Distance_InCell.push_back(R);	
	Det_InCell.push_back(m_ChiNu->DetectorNumber[i]); 
	T_InCell.push_back(m_ChiNu->Time[i]);

	double T_stop = (m_ChiNu->Time[i]); 
	neutron->SetBeta(  (Rdet/T_stop) / c_light);
	
	E_InCell.push_back(m_ChiNu->Energy[i]);
	Elab_InCell.push_back(neutron->GetEnergy());

	double HitX = InteractionCoordinates->GetDetectedPositionX(j);
	double HitY = InteractionCoordinates->GetDetectedPositionY(j);
	double HitZ = InteractionCoordinates->GetDetectedPositionZ(j);
	double HitE = InteractionCoordinates->GetEnergy(j);
	double HitT = InteractionCoordinates->GetTime(j);

	double DetectorX = DetPos.X();
	double DetectorY = DetPos.Y();
	double DetectorZ = DetPos.Z();
	R = DetPos.Mag();

	TVector3 Xaxis(DetectorZ*1./R,0,-DetectorX*1./R);
	TVector3 Yaxis(DetectorY*1./R*Xaxis.Z() , DetectorZ*1./R*Xaxis.X() - DetectorX*1./R*Xaxis.Z(), Xaxis.X()*DetectorY*1./R );

	// scalar product to get the vector projection along the central detector axis:
	double InCellDepth = ( (HitX-DetectorX)*DetectorX + (HitY-DetectorY)*DetectorY + (HitZ-DetectorZ)*DetectorZ ) *1./R; 
	double uProjX      = DetectorX * (1 + InCellDepth*1./R); // Projection of hit point on the detector central axis, axis X
	double uProjY      = DetectorY * (1 + InCellDepth*1./R); // Projection of hit point on the detector central axis, axis Y
	double uProjZ      = DetectorZ * (1 + InCellDepth*1./R); // Projection of hit point on the detector central axis, axis Z

	// Radial distance of the hit point to the detector central axis:
	double InCellRadius = sqrt( pow(HitX - uProjX,2) + pow(HitY - uProjY,2) + pow(HitZ - uProjZ,2) );
	double InCellHit_X = ( (HitX-DetectorX)*Xaxis.X() + (HitY-DetectorY)*Xaxis.Y() + (HitZ-DetectorZ)*Xaxis.Z() ); 
	double InCellHit_Y = ( (HitX-DetectorX)*Yaxis.X() + (HitY-DetectorY)*Yaxis.Y() + (HitZ-DetectorZ)*Yaxis.Z() );

	InCellHitDepth.push_back(InCellDepth);
	InCellHitRadius.push_back(InCellRadius);
	InCellHitEnergy.push_back(HitE);
	InCellHitTime.push_back(HitT);
	InCellHitX.push_back(InCellHit_X);
	InCellHitY.push_back(InCellHit_Y);

	neutron->SetBeta( (R/HitT) / c_light ); 
	InCellHitEnToF.push_back(neutron->GetEnergy());
      }
  }
}

///////////////////////////////////////////////////////////////////////////////
void Analysis::InitOutputBranch() {
  RootOutput::getInstance()->GetTree()->Branch("Einit",&Einit,"Einit/D");
  RootOutput::getInstance()->GetTree()->Branch("ThetaLab",&ThetaLab);
  RootOutput::getInstance()->GetTree()->Branch("Elab",&Elab);   
  RootOutput::getInstance()->GetTree()->Branch("E",&E);   
  RootOutput::getInstance()->GetTree()->Branch("Ex",&Ex);   
  RootOutput::getInstance()->GetTree()->Branch("T",&T);   
  RootOutput::getInstance()->GetTree()->Branch("Distance",&Distance);   
  RootOutput::getInstance()->GetTree()->Branch("Det",&Det);   

  // InCellPosition
  RootOutput::getInstance()->GetTree()->Branch("Elab_InCell",&Elab_InCell);
  RootOutput::getInstance()->GetTree()->Branch("E_InCell",&E_InCell);
  RootOutput::getInstance()->GetTree()->Branch("T_InCell",&T_InCell);
  RootOutput::getInstance()->GetTree()->Branch("Distance_InCell",&Distance_InCell);
  RootOutput::getInstance()->GetTree()->Branch("Det_InCell",&Det_InCell);

  RootOutput::getInstance()->GetTree()->Branch("InCellHitMultiplicity",&InCellHitMultiplicity);
  RootOutput::getInstance()->GetTree()->Branch("InCellHitDepth",&InCellHitDepth);
  RootOutput::getInstance()->GetTree()->Branch("InCellHitRadius",&InCellHitRadius);
  RootOutput::getInstance()->GetTree()->Branch("InCellHitEnergy",&InCellHitEnergy);
  RootOutput::getInstance()->GetTree()->Branch("InCellHitTime",&InCellHitTime);
  RootOutput::getInstance()->GetTree()->Branch("InCellHitEnToF",&InCellHitEnToF);
  RootOutput::getInstance()->GetTree()->Branch("InCellHitX",&InCellHitX);
  RootOutput::getInstance()->GetTree()->Branch("InCellHitY",&InCellHitY);
}

////////////////////////////////////////////////////////////////////////////////
void Analysis::InitInputBranch(){
  RootInput:: getInstance()->GetChain()->SetBranchStatus("InitialConditions",true );
  RootInput:: getInstance()->GetChain()->SetBranchAddress("InitialConditions",&InitialConditions);

  RootInput:: getInstance()->GetChain()->SetBranchStatus("ReactionConditions",true );
  RootInput:: getInstance()->GetChain()->SetBranchAddress("ReactionConditions",&ReactionConditions);
  
  RootInput:: getInstance()->GetChain()->SetBranchStatus("InteractionCoordinates",true );
  RootInput:: getInstance()->GetChain()->SetBranchAddress("InteractionCoordinates",&InteractionCoordinates);
}

////////////////////////////////////////////////////////////////////////////////     
void Analysis::ReInitValue(){
  Einit      = -100;
  Ex.clear();
  ThetaLab.clear();
  Elab.clear();
  E.clear();
  T.clear();
  Distance.clear();
  Det.clear();
  
  Elab_InCell.clear();
  E_InCell.clear();
  T_InCell.clear();
  Distance_InCell.clear();
  Det_InCell.clear();

  InCellHitDepth.clear();
  InCellHitRadius.clear();
  InCellHitEnergy.clear();
  InCellHitTime.clear();
  InCellHitEnToF.clear();
  InCellHitX.clear();
  InCellHitY.clear();
}

////////////////////////////////////////////////////////////////////////////////
void Analysis::End(){
}


////////////////////////////////////////////////////////////////////////////////
//            Construct Method to be pass to the DetectorFactory              //
////////////////////////////////////////////////////////////////////////////////
NPL::VAnalysis* Analysis::Construct(){
  return (NPL::VAnalysis*) new Analysis();
}

////////////////////////////////////////////////////////////////////////////////
//            Registering the construct method to the factory                 //
////////////////////////////////////////////////////////////////////////////////
extern "C"{
  class proxy{
    public:
      proxy(){
        NPL::AnalysisFactory::getInstance()->SetConstructor(Analysis::Construct);
      }
  };

  proxy p;
}

