#!/usr/bin/python3
import os
import subprocess
import numpy as np


def format_value(value):
    """ Format the value by replacing '-' with 'm' and '.' with '_' """
    return str(value).replace('-', 'm').replace('.', '_')

def modify_files(params, outputFileName, outputDir, beamFile,macroFile):
    # Default parameters
    defaults = {'thetaTarget': 0,'offsetTargetX': 0,'offsetTargetY' :0,
        'offsetBeamX' : 0, 'offsetBeamY' : 0, 'sigmaBeamX': 0, 'sigmaBeamY': 0,
    'holderShift' : 0 }

    # Update params with any defaults that aren't specified
    for key, value in defaults.items():
        params.setdefault(key, value)

    # Define file paths
    nptool_path = os.getenv('NPTOOL')
    edin_file = nptool_path + "/NPSimulation/Detectors/EdinburghDSSD/EdinburghDSSD.cc"
    beam_file = "./Beam/" + beamFile
    routine_file = "./routine.sh"

    # Function to preserve indentation
    def replace_line(line, new_content):
        leading_spaces = len(line) - len(line.strip())
        return ' ' * leading_spaces + new_content + '\n'

    # Read and modify EdinburghDSSD.cc
    #with open(edin_file, 'r') as file:
    #    lines = file.readlines()
    #with open(edin_file, 'w') as file:

    # Read and modify beam file
    with open(beam_file, 'r') as file:
        lines = file.readlines()
    with open(beam_file, 'w') as file:
        for line in lines:
            if "x0=" in line:
                new_line = replace_line(line, f"x0= {params['offsetBeamX']:.1f}")
                file.write(new_line[1:])
            elif "y0=" in line:
                new_line = replace_line(line, f"y0= {params['offsetBeamY']:.1f}")
                file.write(new_line[1:])
            elif "sigmaX =" in line:
                new_line = replace_line(line, f"sigmaX = {params['sigmaBeamX']:.2f}")
                file.write(new_line[1:])
            elif "sigmaY =" in line:
                new_line = replace_line(line, f"sigmaY = {params['sigmaBeamY']:.2f}")
                file.write(new_line[1:])
            else:
                file.write(line)

    # Read and modify routine.sh
    with open(routine_file, 'r') as file:
        lines = file.readlines()
    with open(routine_file, 'w') as file:
        for line in lines:
            if 'SIM_FILE=' in line:
                parts = []
                for param, value in params.items():
                    if value != defaults[param]:
                        format_val = format_value(value)
                        parts.append(f"{param}{format_val}")
                new_sim_file = outputFileName + '__'.join(parts)
                new_line = replace_line(line, f'SIM_FILE="{new_sim_file}"')
                file.write(new_line[1:])
            elif 'DIR=' in line :
                new_line = replace_line(line, 'DIR="'+outputDir + '"')
                file.write(new_line[1:])
            elif 'MACRO=' in line:
                new_line = replace_line(line, 'MACRO="'+macroFile + '"')
                file.write(new_line[1:])
            elif 'EVENT_SOURCE=' in line :
                new_line = replace_line(line, 'EVENT_SOURCE="'+beamFile + '"')
                file.write(new_line[1:])
            else:
                file.write(line)

outputFileName = "222Ra_alpha__1e7__"
outputDir = "nice_plot"
beamFile = "alpha_222Ra.source"
macroFile = "run2e6"

params = {'offsetTargetX': 0, 'offsetTargetY': 0,
    'offsetBeamX' : 0,'offsetBeamY' : 0, 'thetaTarget' : 0,
'sigmaBeamX' : 0, 'sigmaBeamY' : 0 }




xBeam = -6
yBeam = 6
radius_target = 6
sigmaBeam = 0.2
holder_shift = 0.5
theta = 0


for offsetX in [-3]:

    offsetY = yBeam - (offsetX-xBeam)*np.tan(theta*np.pi/180)
    if(np.sqrt((offsetY-yBeam)**2+(offsetX-xBeam)**2))>radius_target:
        print("BEAM OUTSIDE TARGET ! ")
    params = {'offsetTargetX': offsetX, 'offsetTargetY': offsetY,
            'offsetBeamX' : xBeam,'offsetBeamY' : yBeam, 'thetaTarget' : theta,
            'sigmaBeamX' : sigmaBeam, 'sigmaBeamY' : sigmaBeam,
             'holderShift' : holder_shift }
    #print(params)
    modify_files(params, outputFileName,outputDir, beamFile, macroFile)
    subprocess.run('./routine.sh', shell=True, executable="/bin/bash", check=True)
