/*****************************************************************************
 * Copyright (C) 2009-2016    this file is part of the NPTool Project        *
 *                                                                           *
 * For the licensing terms see $NPTOOL/Licence/NPTool_Licence                *
 * For the list of contributors see $NPTOOL/Licence/Contributors             *
 *****************************************************************************/

/*****************************************************************************
 * Original Author: XAUTHORX  contact address: XMAILX                        *
 *                                                                           *
 * Creation Date  : XMONTHX XYEARX                                           *
 * Last update    :                                                          *
 *---------------------------------------------------------------------------*
 * Decription:                                                               *
 *  This class describe  TOGAXSI_SI analysis project                       *
 *                                                                           *
 *---------------------------------------------------------------------------*
 * Comment:                                                                  *
 *                                                                           *
 *****************************************************************************/

#include<iostream>
using namespace std;
#include"Analysis.h"
#include"NPAnalysisFactory.h"
#include"NPDetectorManager.h"
#include "NPFunction.h"
#include "NPOptionManager.h"
#include "NPPhysicalConstants.h"
#include "NPTrackingUtility.h"
#include "TRandom3.h"

#include "Math/Plane3D.h"

using namespace ROOT::Math;

////////////////////////////////////////////////////////////////////////////////
Analysis::Analysis(){
}
////////////////////////////////////////////////////////////////////////////////
Analysis::~Analysis(){
}

////////////////////////////////////////////////////////////////////////////////
void Analysis::Init(){
   IC = new TInitialConditions;
   DC = new TInteractionCoordinates;
   RC = new TReactionConditions;

   InitOutputBranch();
   InitInputBranch();

   TOGAXSI_SI= (TTOGAXSI_SIPhysics*) m_DetectorManager->GetDetector("TOGAXSI_SI");
   
   m_QFS = new NPL::QFS();
   m_QFS->ReadConfigurationFile(NPOptionManager::getInstance()->GetReactionFile());

   myBeam = new NPL::Beam();
   myBeam->ReadConfigurationFile(NPOptionManager::getInstance()->GetReactionFile());
   InitialBeamEnergy = myBeam->GetEnergy();

   TargetThickness = 150*mm;
   string TargetMaterial = "LH2";
   //Energyloss Tables
   string BeamName = NPL::ChangeNameToG4Standard(myBeam->GetName());

   rand = new TRandom3();
}

////////////////////////////////////////////////////////////////////////////////
void Analysis::TreatEvent(){

   ReInitValue();

   bool perfectBeamEnergy = false;
   bool perfectBeamTracking = false;
   bool perfectProtonTracking = true;
   bool perfectProtonEnergy = true;

   unsigned int size = TOGAXSI_SI->GetEventMultiplicity();
   unsigned int size_cluster = TOGAXSI_SI->GetEventMultiplicity_cluster(); 

   double r_1 = 60; // Inner Z radius
   double r_2 = 66; // Inner X radius
   double r_3 = 90; // Outer Z radius
   double r_4 = 96;  // Outer X radius

   bool threeplanes_p2p = false;
   bool fourplanes_p2p = true;

   bool cluster_fourplanes = false;
   bool cluster_threeplanes = false;
   
   // Calculate z position on x-tracker for particle 1 and 2

   if (threeplanes_p2p == true) {
     /////////Protons (E,P) Reconstruction /////////

     //Particle 1
//     TVector3 InnerPosX1 = TOGAXSI_SI->GetInnerXPositionOfInteraction(0);
//     TVector3 InnerPosZ1 = TOGAXSI_SI->GetInnerZPositionOfInteraction(0);
//     TVector3 OuterPosX1 = TOGAXSI_SI->GetOuterXPositionOfInteraction(0);
//     TVector3 OuterPosZ1 = TOGAXSI_SI->GetOuterZPositionOfInteraction(0);

     //Particle 2
//     TVector3 InnerPosX2 = TOGAXSI_SI->GetInnerXPositionOfInteraction(1);
//     TVector3 InnerPosZ2 = TOGAXSI_SI->GetInnerZPositionOfInteraction(1);
//     TVector3 OuterPosX2 = TOGAXSI_SI->GetOuterXPositionOfInteraction(1);
//     TVector3 OuterPosZ2 = TOGAXSI_SI->GetOuterZPositionOfInteraction(1);

     //Test
     //Particle 1
     TVector3 InnerPosX1(0,66,0);
     TVector3 InnerPosZ1(0,60,40);
//     TVector3 OuterPosX1(10,96,0);
     TVector3 OuterPosX1(27,96,0);
     TVector3 OuterPosZ1(0,90,70);

     //Particle 2
     TVector3 InnerPosX2(0,-66,0);
     TVector3 InnerPosZ2(0,-60,50);
//     TVector3 OuterPosX2(20,-96,0);
     TVector3 OuterPosX2(-14,-96,0);
     TVector3 OuterPosZ2(0,-90,70);


     //Particle 1
     double z_4 = (r_4 - r_1) / (r_3 - r_1) * (OuterPosZ1.Z() - InnerPosZ1.Z()) + InnerPosZ1.Z();
     cout << OuterPosX1.Z() << endl;
     OuterPosX1.SetZ(z_4);    
     cout << OuterPosX1.Z() << endl;
   
     //Particle 2
     z_4 = (r_4 - r_1) / (r_3 - r_1) * (OuterPosZ2.Z() - InnerPosZ2.Z()) + InnerPosZ2.Z();
     cout << OuterPosX2.Z() << endl;
     OuterPosX2.SetZ(z_4);    
     cout << OuterPosX2.Z() << endl;

     // Determine the plane for Particle 1 and 2
     // Particle 1
     double phi1 = TMath::ACos( InnerPosZ1.Y() / r_1 );
     TVector3 PlanePoint1(5,r_1,InnerPosZ1.Z());  
     TVector3 PlanePoint2(-5,r_3,OuterPosZ1.Z());  

     cout << PlanePoint1.Y() << endl;
     cout << PlanePoint1.Z() << endl;
     cout << "Phi1: " << phi1 << endl;

     PlanePoint1.RotateZ(phi1);
     PlanePoint2.RotateZ(phi1);

     TVector3 normalVector1;
     TVector3 directionVector1 = OuterPosX1 - PlanePoint1;
     TVector3 directionVector2 = OuterPosX1 - PlanePoint2;
     normalVector1 = directionVector1.Cross(directionVector2);
   
     cout << normalVector1.x() << endl;
     cout << normalVector1.y() << endl;
     cout << normalVector1.z() << endl;

//     double a_1 = normalVector1.x();
//     double b_1 = normalVector1.y();
//     double c_1 = normalVector1.z();
//     double d_1 = -1 * normalVector1 * OuterPosX1;
//     ROOT::Math::Plane3D Plane1(normalVector1, OuterPosX1); 
//     ROOT::Math::Plane3D Plane1(a_1,b_1,c_1,d_1); 

     // Particle 2
     double phi2 = TMath::ACos( InnerPosZ2.Y() / r_1 );
     TVector3 PlanePoint3(10,r_1,InnerPosZ2.Z());  
     TVector3 PlanePoint4(-9,r_1,InnerPosZ2.Z());  

     cout << "Phi2: " << phi2 << endl;
     cout << PlanePoint3.x() << endl;
     cout << PlanePoint3.y() << endl;
     cout << PlanePoint3.z() << endl;


     PlanePoint3.RotateZ(phi2);
     PlanePoint4.RotateZ(phi2);

     TVector3 normalVector2;
     directionVector1 = OuterPosX2 - PlanePoint3;
     directionVector2 = OuterPosX2 - PlanePoint4;
     normalVector2 = directionVector1.Cross(directionVector2);
  
     cout << normalVector2.x() << endl;
     cout << normalVector2.y() << endl;
     cout << normalVector2.z() << endl;

//     double a_2 = normalVector2.x();
//     double b_2 = normalVector2.y();
//     double c_2 = normalVector2.z();
//     double d_2 = -1 * normalVector2 * OuterPosX2;
//     Plane3D Plane2(normalVector2, OuterPosX2); 
//     ROOT::Math::Plane3D Plane2(a_2,b_2,c_2,d_2); 

     TVector3 line_position;
     TVector3 line_direction;

     CalcLine(normalVector1, OuterPosX1, normalVector2, OuterPosX2, line_position, line_direction);

     cout << "line Point X" << line_position.X() << endl;
     cout << line_position.Y() << endl;
     cout << line_position.Z() << endl;
     cout << "line direction Z" << line_direction.X() << endl;
     cout << line_direction.Y() << endl;
     cout << line_direction.Z() << endl;

    //Calculate points from line and target cylinder
    //Cylinder 
    double r_c = 7.5;
 
    // Solve quadratic equation a_1* x^2 + a_2 *x + a_3 = 0
    double a_1 = pow(line_direction.x(),2.) + pow(line_direction.y(),2.);
    double a_2 = 2 * line_direction.x() * line_position.x() + 2 * line_direction.y() * line_position.y();    
    double a_3 = pow(line_position.x(),2.) + pow(line_position.y(),2.) - r_c * r_c;
     
    //solutions
    double t_1 = (-a_2 + sqrt(a_2*a_2 - 4 * a_1 * a_3) ) / (2 * a_1);
    double t_2 = (-a_2 - sqrt(a_2*a_2 - 4 * a_1 * a_3) ) / (2 * a_1);

    //Calculate coordinates of the two solutions
    double z_1 = line_position.z() + t_1 * line_direction.z();
    double z_2 = line_position.z() + t_2 * line_direction.z();

    double x_1 = line_position.x() + t_1 * line_direction.x();
    double x_2 = line_position.x() + t_2 * line_direction.x(); 
    double y_1 = line_position.y() + t_1 * line_direction.y();
    double y_2 = line_position.y() + t_2 * line_direction.y(); 


    //Check if one of the point is in the target and reconstruct vertex
    double z_min = 50;
    double z_max = 100;

    if ( (z_1 > z_min && z_1 < z_max) || (z_2 > z_min && z_2 < z_max) ) {
      VertexZ = z_2 - z_1;
      VertexX = x_2 - x_1;
      VertexY = y_2 - y_1;
    } else {
      VertexZ = -9999.;
      VertexX = -9999.;
      VertexY = -9999.;
    }


  } else if ( fourplanes_p2p == true ) {
    if (size==2) {
//      cout << "test" << endl;
      

      //Particle 1
      TVector3 InnerPosX1 = TOGAXSI_SI->GetInnerXPositionOfInteraction(0);
      TVector3 InnerPosZ1 = TOGAXSI_SI->GetInnerZPositionOfInteraction(0);
      TVector3 OuterPosX1 = TOGAXSI_SI->GetOuterXPositionOfInteraction(0);
      TVector3 OuterPosZ1 = TOGAXSI_SI->GetOuterZPositionOfInteraction(0);

      //Particle 2
      TVector3 InnerPosX2 = TOGAXSI_SI->GetInnerXPositionOfInteraction(1);
      TVector3 InnerPosZ2 = TOGAXSI_SI->GetInnerZPositionOfInteraction(1);
      TVector3 OuterPosX2 = TOGAXSI_SI->GetOuterXPositionOfInteraction(1);
      TVector3 OuterPosZ2 = TOGAXSI_SI->GetOuterZPositionOfInteraction(1);

//      cout <<"______Event Multiplicty 2______" << endl;

      //Particle 1

      double z_2 = (r_2 - r_1) / (r_3 - r_1) * (OuterPosZ1.Z() - InnerPosZ1.Z()) + InnerPosZ1.Z();
      InnerPosX1.SetZ(z_2); 

      double z_4 = (r_4 - r_1) / (r_3 - r_1) * (OuterPosZ1.Z() - InnerPosZ1.Z()) + InnerPosZ1.Z();
      OuterPosX1.SetZ(z_4);    


      double x_1 = (r_1 - r_2) / (r_4 - r_2) * (OuterPosX1.X() - InnerPosX1.X()) + InnerPosX1.X();
      InnerPosZ1.SetX(x_1);

      double x_3 = (r_3 - r_2) / (r_4 - r_2) * (OuterPosX1.X() - InnerPosX1.X()) + InnerPosX1.X();
      OuterPosZ1.SetX(x_3);
/*
      cout << "InnerPos Z Particle 1: " << InnerPosZ1.Z() << endl; 
      cout << "InnerPos X Particle 1: " << InnerPosX1.Z() << endl; 
      cout << "OuterPos Z Particle 1: " << OuterPosZ1.Z() << endl; 
      cout << "OuterPos X Particle 1: " << OuterPosX1.Z() << endl;
      cout << "InnerPos Z Particle 1 X: " << InnerPosZ1.X() << endl; 
      cout << "InnerPos X Particle 1 X: " << InnerPosX1.X() << endl; 
      cout << "OuterPos Z Particle 1 X: " << OuterPosZ1.X() << endl; 
      cout << "OuterPos X Particle 1 X: " << OuterPosX1.X() << endl;
*/    

      //Particle 2
      z_2 = (r_2 - r_1) / (r_3 - r_1) * (OuterPosZ2.Z() - InnerPosZ2.Z()) + InnerPosZ2.Z();
      InnerPosX2.SetZ(z_2); 
      z_4 = (r_4 - r_1) / (r_3 - r_1) * (OuterPosZ2.Z() - InnerPosZ2.Z()) + InnerPosZ2.Z();
      OuterPosX2.SetZ(z_4);    
      
      x_1 = (r_1 - r_2) / (r_4 - r_2) * (OuterPosX2.X() - InnerPosX2.X()) + InnerPosX2.X();
      InnerPosZ2.SetX(x_1);
      x_3 = (r_3 - r_2) / (r_4 - r_2) * (OuterPosX2.X() - InnerPosX2.X()) + InnerPosX2.X();
      OuterPosZ2.SetX(x_3);

      
      //      cout << OuterPosX2.Z() << endl;



      TVector3 Vertex;
      TVector3 delta;
      Distance = NPL::MinimumDistanceTwoLines(InnerPosX1, OuterPosX1, InnerPosX2, OuterPosX2, Vertex, delta);     

      VertexX = Vertex.X();
      VertexY = Vertex.Y();
      VertexZ = Vertex.Z();
      deltaX = delta.X();
      deltaY = delta.Y();
      deltaZ = delta.Z();

      SiInnerZ1_X = InnerPosZ1.X();
      SiInnerZ1_Y = InnerPosZ1.Y();
      SiInnerZ1_Z = InnerPosZ1.Z();
      SiOuterZ1_X = OuterPosZ1.X();
      SiOuterZ1_Y = OuterPosZ1.Y();
      SiOuterZ1_Z = OuterPosZ1.Z();

      SiInnerX1_X = InnerPosX1.X();
      SiInnerX1_Y = InnerPosX1.Y();
      SiInnerX1_Z = InnerPosX1.Z();


      SiInnerZ2_X = InnerPosZ2.X();
      SiInnerZ2_Y = InnerPosZ2.Y();
      SiInnerZ2_Z = InnerPosZ2.Z();
      SiOuterZ2_X = OuterPosZ2.X();
      SiOuterZ2_Y = OuterPosZ2.Y();
      SiOuterZ2_Z = OuterPosZ2.Z();

      SiInnerX2_X = InnerPosX2.X();
      SiInnerX2_Y = InnerPosX2.Y();
      SiInnerX2_Z = InnerPosX2.Z();

      cout <<"InnerZ X1: " << SiInnerZ1_X << endl;
      cout <<"InnerZ Z1: " << SiInnerZ1_Z << endl;
      cout <<"OuterZ X1: " << SiOuterZ1_X << endl;
      cout <<"OuterZ Z1: " << SiOuterZ1_Z << endl;

      cout <<"InnerX X1: " << SiInnerX1_X << endl;
      cout <<"InnerX Z1: " << SiInnerX1_Z << endl;
      
      cout <<"InnerZ X2: " << SiInnerZ2_X << endl;
      cout <<"InnerZ Z2: " << SiInnerZ2_Z << endl;
      cout <<"OuterZ X2: " << SiOuterZ2_X << endl;
      cout <<"OuterZ Z2: " << SiOuterZ2_Z << endl;

      cout <<"InnerX X2: " << SiInnerX2_X << endl;
      cout <<"InnerX Z2: " << SiInnerX2_Z << endl;

      cout << DC->GetDetectedPositionX(0) << endl;
      cout << DC->GetDetectedPositionX(1) << endl;
      cout << DC->GetDetectedPositionX(2) << endl;
      cout << DC->GetDetectedPositionX(3) << endl;
      cout << DC->GetDetectedPositionX(4) << endl;
      cout << DC->GetDetectedPositionX(5) << endl;
      cout << DC->GetDetectedPositionX(6) << endl;
      cout << DC->GetDetectedPositionX(7) << endl;

      cout << DC->GetDetectedPositionZ(0) << endl;
      cout << DC->GetDetectedPositionZ(1) << endl;
      cout << DC->GetDetectedPositionZ(2) << endl;
      cout << DC->GetDetectedPositionZ(3) << endl;
      cout << DC->GetDetectedPositionZ(4) << endl;
      cout << DC->GetDetectedPositionZ(5) << endl;
      cout << DC->GetDetectedPositionZ(6) << endl;
      cout << DC->GetDetectedPositionZ(7) << endl;


      cout <<"Targetcell: " << DC->GetDetectedPositionZ(8) << endl;
      cout <<"Targetcell: " << DC->GetDetectedPositionZ(9) << endl;
      cout <<"Targetcell: " << DC->GetDetectedPositionZ(10) << endl;
      cout <<"Targetcell: " << DC->GetDetectedPositionZ(11) << endl;
      cout <<"Targetcell: " << DC->GetDetectedPositionZ(12) << endl;
      cout <<"Targetcell: " << DC->GetDetectedPositionZ(13) << endl;
      cout <<"Targetcell: " << DC->GetDetectedPositionZ(14) << endl;
      cout <<"Targetcell: " << DC->GetDetectedPositionZ(15) << endl;
      cout <<"Targetcell: " << DC->GetDetectedPositionZ(16) << endl;
      cout <<"Targetcell: " << DC->GetDetectedPositionZ(17) << endl;
      cout <<"Targetcell: " << DC->GetDetectedPositionZ(18) << endl;
      cout <<"Targetcell: " << DC->GetDetectedPositionZ(19) << endl;
      cout <<"Targetcell: " << DC->GetDetectedPositionZ(20) << endl;
      cout <<"Targetcell: " << DC->GetDetectedPositionZ(21) << endl;
      cout <<"Targetcell: " << DC->GetDetectedPositionZ(22) << endl;
      cout <<"Targetcell: " << DC->GetDetectedPositionZ(23) << endl;
      cout <<"Targetcell: " << DC->GetDetectedPositionZ(24) << endl;
      cout <<"Targetcell: " << DC->GetDetectedPositionZ(25) << endl;
      cout <<"Targetcell: " << DC->GetDetectedPositionZ(26) << endl;
      cout <<"Targetcell: " << DC->GetDetectedPositionZ(27) << endl;
      cout <<"Targetcell: " << DC->GetDetectedPositionZ(28) << endl;
      cout <<"Targetcell: " << DC->GetDetectedPositionZ(29) << endl;
      cout <<"Targetcell: " << DC->GetDetectedPositionZ(30) << endl;
      cout <<"Targetcell: " << DC->GetDetectedPositionZ(31) << endl;
      cout <<"Targetcell: " << DC->GetDetectedPositionZ(32) << endl;
      cout <<"Targetcell: " << DC->GetDetectedPositionZ(33) << endl;
      cout <<"Targetcell: " << DC->GetDetectedPositionZ(34) << endl;
      cout <<"Targetcell: " << DC->GetDetectedPositionZ(35) << endl;
      cout <<"Targetcell: " << DC->GetDetectedPositionZ(36) << endl;
      cout <<"Targetcell: " << DC->GetDetectedPositionZ(37) << endl;
      cout <<"Targetcell: " << DC->GetDetectedPositionZ(38) << endl;
      cout <<"Targetcell: " << DC->GetDetectedPositionZ(39) << endl;
      cout <<"Targetcell: " << DC->GetDetectedPositionZ(40) << endl;
      cout <<"Targetcell: " << DC->GetDetectedPositionZ(41) << endl;


      cout <<"Targetcell X: " << DC->GetDetectedPositionX(10) << endl;
      cout <<"Targetcell X: " << DC->GetDetectedPositionX(11) << endl;
      cout <<"Targetcell X: " << DC->GetDetectedPositionX(12) << endl;
      cout <<"Targetcell X: " << DC->GetDetectedPositionX(13) << endl;
      cout <<"Targetcell X: " << DC->GetDetectedPositionX(14) << endl;
      cout <<"Targetcell X: " << DC->GetDetectedPositionX(15) << endl;
      cout <<"Targetcell X: " << DC->GetDetectedPositionX(16) << endl;
      cout <<"Targetcell X: " << DC->GetDetectedPositionX(17) << endl;
      cout <<"Targetcell Y: " << DC->GetDetectedPositionY(10) << endl;
      cout <<"Targetcell Y: " << DC->GetDetectedPositionY(11) << endl;
      cout <<"Targetcell Y: " << DC->GetDetectedPositionY(12) << endl;
      cout <<"Targetcell Y: " << DC->GetDetectedPositionY(13) << endl;
      cout <<"Targetcell Y: " << DC->GetDetectedPositionY(14) << endl;
      cout <<"Targetcell Y: " << DC->GetDetectedPositionY(15) << endl;
      cout <<"Targetcell Y: " << DC->GetDetectedPositionY(16) << endl;
      cout <<"Targetcell Y: " << DC->GetDetectedPositionY(17) << endl;

      cout <<"Targetcell X: " << DC->GetDetectedPositionX(20) << endl;
      cout <<"Targetcell X: " << DC->GetDetectedPositionX(21) << endl;
      cout <<"Targetcell X: " << DC->GetDetectedPositionX(22) << endl;
      cout <<"Targetcell X: " << DC->GetDetectedPositionX(23) << endl;
      cout <<"Targetcell X: " << DC->GetDetectedPositionX(24) << endl;
      cout <<"Targetcell X: " << DC->GetDetectedPositionX(25) << endl;
      cout <<"Targetcell X: " << DC->GetDetectedPositionX(26) << endl;
      cout <<"Targetcell X: " << DC->GetDetectedPositionX(27) << endl;
      cout <<"Targetcell Y: " << DC->GetDetectedPositionY(20) << endl;
      cout <<"Targetcell Y: " << DC->GetDetectedPositionY(21) << endl;
      cout <<"Targetcell Y: " << DC->GetDetectedPositionY(22) << endl;
      cout <<"Targetcell Y: " << DC->GetDetectedPositionY(23) << endl;
      cout <<"Targetcell Y: " << DC->GetDetectedPositionY(24) << endl;
      cout <<"Targetcell Y: " << DC->GetDetectedPositionY(25) << endl;
      cout <<"Targetcell Y: " << DC->GetDetectedPositionY(26) << endl;
      cout <<"Targetcell Y: " << DC->GetDetectedPositionY(27) << endl;

      cout << DC->GetDetectedPositionY(0) << endl;
      cout << DC->GetDetectedPositionY(1) << endl;
      cout << DC->GetDetectedPositionY(2) << endl;
      cout << DC->GetDetectedPositionY(3) << endl;
      cout << DC->GetDetectedPositionY(4) << endl;
      cout << DC->GetDetectedPositionY(5) << endl;
      cout << DC->GetDetectedPositionY(6) << endl;
      cout << DC->GetDetectedPositionY(7) << endl;


      SiInnerZ1_X_sim = DC->GetDetectedPositionX(3);
      SiInnerZ1_Y_sim = DC->GetDetectedPositionY(3); 
      SiInnerZ1_Z_sim = DC->GetDetectedPositionZ(3);
      SiOuterZ1_X_sim = DC->GetDetectedPositionX(7);
      SiOuterZ1_Y_sim = DC->GetDetectedPositionY(7);
      SiOuterZ1_Z_sim = DC->GetDetectedPositionZ(7);

      SiInnerX1_X_sim = DC->GetDetectedPositionX(1);
      SiInnerX1_Y_sim = DC->GetDetectedPositionY(1); 
      SiInnerX1_Z_sim = DC->GetDetectedPositionZ(1);

      InnerZ1_theta_sim = DC->GetDetectedAngleTheta(3);
      OuterZ1_theta_sim = DC->GetDetectedAngleTheta(7);
      InnerX1_theta_sim = DC->GetDetectedAngleTheta(1);


/*
      cout <<"InnerZ X1 sim: " << SiInnerZ1_X_sim << endl;
      cout <<"InnerZ Z1 sim: " << SiInnerZ1_Z_sim << endl;
      cout <<"OuterZ X1 sim: " << SiOuterZ1_X_sim << endl;
      cout <<"OuterZ Z1 sim: " << SiOuterZ1_Z_sim << endl;

      cout <<"InnerX X1 sim: " << SiInnerX1_X_sim << endl;
      cout <<"InnerX Z1 sim: " << SiInnerX1_Z_sim << endl;

      cout <<"_______ Event Multiplicity 2 end_________" << endl;
*/     
      theta_vertex_1_sim = RC->GetTheta(0);
      theta_vertex_2_sim = RC->GetTheta(1);

     if(perfectProtonTracking) {
       // From Reaction Conditions
       VertexX_sim = RC->GetVertexPositionX();
       VertexY_sim = RC->GetVertexPositionY();
       VertexZ_sim = RC->GetVertexPositionZ();
       //Proton2sim = RC->GetParticleDirection(0).Unit();
       //Proton1 = RC->GetParticleDirection(1).Unit();
     }

     TVector3 direction_cal(SiInnerZ1_X_sim - VertexX_sim, SiInnerZ1_Y_sim - VertexY_sim, SiInnerZ1_Z_sim - VertexZ_sim);
     direction_cal = direction_cal.Unit();

      if (RC->GetTheta(0) < 45.01 && RC->GetTheta(0) > 44.99) {
      cout << "________________Test_______________" << endl;
      cout << RC->GetTheta(0) << endl;
      cout << RC->GetTheta(1) << endl;

      cout << RC->GetParticleDirection(0).Theta() * 180/TMath::Pi() << endl;
      cout << RC->GetParticleDirection(1).Theta() * 180/TMath::Pi() << endl;

      cout << "Particle direction X: " << RC->GetParticleDirection(0).X() << endl;
      cout << "Particle direction Y: " << RC->GetParticleDirection(0).Y() << endl;
      cout << "Particle direction Z: " << RC->GetParticleDirection(0).Z() << endl;

      cout << "Particle direction X: " << RC->GetParticleDirection(1).X() << endl;
      cout << "Particle direction Y: " << RC->GetParticleDirection(1).Y() << endl;
      cout << "Particle direction Z: " << RC->GetParticleDirection(1).Z() << endl;

      cout << "Particle direction X calculated: " << direction_cal.X() << endl;
      cout << "Particle direction Y calculated: " << direction_cal.Y() << endl;
      cout << "Particle direction Z calculated: " << direction_cal.Z() << endl;

      cout << "Calculated Theta: " << direction_cal.Theta() * 180/TMath::Pi() << endl;

      cout << DC->GetDetectedAngleTheta(0) << endl;
      cout << DC->GetDetectedAngleTheta(1) << endl;
      cout << DC->GetDetectedAngleTheta(2) << endl;
      cout << DC->GetDetectedAngleTheta(3) << endl;
      cout << DC->GetDetectedAngleTheta(4) << endl;
      cout << DC->GetDetectedAngleTheta(5) << endl;
      cout << DC->GetDetectedAngleTheta(6) << endl;
      cout << DC->GetDetectedAngleTheta(7) << endl;
      }

    } 




//       VertexX_sim = RC->GetVertexPositionX();
//       VertexY_sim = RC->GetVertexPositionY();
//       VertexZ_sim = RC->GetVertexPositionZ();
    
  } else if (cluster_fourplanes == true) {
    if (size == 1 && size_cluster == 1) {

      //Particle 1
      TVector3 InnerPosX = TOGAXSI_SI->GetInnerXPositionOfInteraction(0);
      TVector3 InnerPosZ = TOGAXSI_SI->GetInnerZPositionOfInteraction(0);
      TVector3 OuterPosX = TOGAXSI_SI->GetOuterXPositionOfInteraction(0);
      TVector3 OuterPosZ = TOGAXSI_SI->GetOuterZPositionOfInteraction(0);

      //Particle 2
      TVector3 Cluster1PosX = TOGAXSI_SI->GetClusterX1PositionOfInteraction(0);
      TVector3 Cluster1PosY = TOGAXSI_SI->GetClusterY1PositionOfInteraction(0);
      TVector3 Cluster2PosX = TOGAXSI_SI->GetClusterX2PositionOfInteraction(0);
      TVector3 Cluster2PosY = TOGAXSI_SI->GetClusterY2PositionOfInteraction(0);

      //Particle 1
      double z_2 = (r_2 - r_1) / (r_3 - r_1) * (OuterPosZ.Z() - InnerPosZ.Z()) + InnerPosZ.Z();
      InnerPosX.SetZ(z_2); 
 
      double z_4 = (r_4 - r_1) / (r_3 - r_1) * (OuterPosZ.Z() - InnerPosZ.Z()) + InnerPosZ.Z();
      OuterPosX.SetZ(z_4);    

      //Particle 2
      double x_2 = (Cluster1PosY.Z() - Cluster1PosX.Z()) / (Cluster2PosX.Z() - Cluster1PosX.Z()) * (Cluster2PosX.X() - Cluster1PosX.X()) + Cluster1PosX.X();
      Cluster1PosY.SetX(x_2); 
 
      double x_4 = (Cluster2PosY.Z() - Cluster1PosX.Z()) / (Cluster2PosX.Z() - Cluster1PosX.Z()) * (Cluster2PosX.X() - Cluster1PosX.X()) + Cluster1PosX.X();
      Cluster2PosY.SetX(x_4);    

/*
      cout << Cluster1PosY.Z() << endl;
      cout << Cluster1PosX.Z() << endl;
      cout << Cluster2PosX.Z() << endl;
      cout << Cluster2PosY.Z() << endl;

      cout << "Z1: " << InnerPosZ.Z() << endl;
      cout << "Z2: " << InnerPosX.Z() << endl;
      cout << "Z3: " << OuterPosZ.Z() << endl;
      cout << "Z4: " << OuterPosX.Z() << endl;
*/
      
      cout << "X1: " << InnerPosZ.X() << endl;
      cout << "X2: " << InnerPosX.X() << endl;
      cout << "X3: " << OuterPosZ.X() << endl;
      cout << "X4: " << OuterPosX.X() << endl;

/*
      cout << "X1: " << Cluster1PosX.X() << endl;
      cout << "X2: " << Cluster1PosY.X() << endl;
      cout << "X3: " << Cluster2PosX.X() << endl;
      cout << "X4: " << Cluster2PosY.X() << endl;
*/

      TVector3 Vertex;
      TVector3 delta;
      Distance = NPL::MinimumDistanceTwoLines(InnerPosX, OuterPosX, Cluster1PosY, Cluster2PosY, Vertex, delta);     

      VertexX = Vertex.X();
      VertexY = Vertex.Y();
      VertexZ = Vertex.Z();
      deltaX = delta.X();
      deltaY = delta.Y();
      deltaZ = delta.Z();


      VertexX_sim = RC->GetVertexPositionX();
      VertexY_sim = RC->GetVertexPositionY();
      VertexZ_sim = RC->GetVertexPositionZ();

    }
  } else if (cluster_threeplanes == true) {
    if (size == 1 && size_cluster == 1) {

      //Particle 1
      TVector3 InnerPosZ = TOGAXSI_SI->GetInnerZPositionOfInteraction(0);
      TVector3 OuterPosX = TOGAXSI_SI->GetOuterXPositionOfInteraction(0);
      TVector3 OuterPosZ = TOGAXSI_SI->GetOuterZPositionOfInteraction(0);

      //Particle 2
      TVector3 Cluster1PosX = TOGAXSI_SI->GetClusterX1PositionOfInteraction(0);
      TVector3 Cluster2PosX = TOGAXSI_SI->GetClusterX2PositionOfInteraction(0);
      TVector3 Cluster2PosY = TOGAXSI_SI->GetClusterY2PositionOfInteraction(0);
    }
  }


}

////////////////////////////////////////////////////////////////////////////////
/*void Analysis::CalcLine(const TVector3& n1, const TVector3& p1, TVector3& n2, TVector3& p2, TVector3& point) {

  direction = n1.Cross(n2);

  double d_1 = -1 * n1 * p1; 
  double d_2 = -1 * n2 * p2;

  TVector3 v1 = n1 * d_2 - n2 * d_1;
  double c1 = n1.Dot(n1.cross(n2));
  point = v1.Cross(direction) * (1.0/ c1);

  return 0;

}
*/
void Analysis::CalcLine(const TVector3& n1, const TVector3& p1, const TVector3& n2, const TVector3& p2, TVector3& point, TVector3& direction) {

  point.Clear();
  direction.Clear();


  direction = n1.Cross(n2);

  double a_1 = n1.x();
  double b_1 = n1.y();
  double c_1 = n1.z();
  double a_2 = n2.x();
  double b_2 = n2.y();
  double c_2 = n2.z();

  double d_1 = n1 * p1;
  double d_2 = n2 * p2;
 

  cout << "n1: " << n1.x() << endl;
  cout << "n1: " << n1.y() << endl;
  cout << "n1: " << n1.z() << endl; 

  cout << "n2: " << n2.x() << endl;
  cout << "n2: " << n2.y() << endl;
  cout << "n2: " << n2.z() << endl; 
  cout << "x: " << p2.x() << endl;
  cout << "y: " << p2.y() << endl;
  cout << "z: " << p2.z() << endl; 

  cout << "d1: " << d_1 << endl;
  cout << "d2: " << d_2 << endl;


  //Calculate Point
  
  if (c_1 != 0 || c_2 != 0) {
    point.SetZ(0);

    point.SetX( (b_2 * d_1 - b_1 * d_2) / (b_2 * a_1 - b_1 * a_2) );
    point.SetY( (a_2 * d_1 - a_1 * d_2) / (a_2 * b_1 - a_1 * b_2) );
  } else if (b_1 != 0 || b_2 != 0 ) {
    point.SetY(0);

    point.SetX( (c_2 * d_1 - c_1 * d_2) / (c_2 * a_1 - c_1 * a_2) );
    point.SetZ( (a_2 * d_1 - a_1 * d_2) / (a_2 * c_1 - a_1 * c_2) );
  } else if (a_1 != 0 ||  a_2 != 0 ) {
    point.SetX(0);

    point.SetY( (c_2 * d_1 - c_1 * d_2) / (c_2 * b_1 - c_1 * b_2) ); 
    point.SetZ( (b_2 * d_1 - b_1 * d_2) / (b_2 * c_1 - b_1 * c_2) );
  }


}

////////////////////////////////////////////////////////////////////////////////
double Analysis::ReconstructProtonEnergy(const TVector3& x0, const TVector3& dir, const double& Ecatana) {
  TVector3 Normal = TVector3(0, 1, 0);
  Normal.SetPhi(dir.Phi());
  double Theta = dir.Angle(Normal);
  // Catana Al housing
  double E = protonAl.EvaluateInitialEnergy(Ecatana, 0.5 * mm, Theta);
  //cout << "Ecatana=" << Ecatana << endl;
  //cout << "Erec0=" << E << endl;

  //Outer Barrel
  E = protonSi.EvaluateInitialEnergy(E, 300 * micrometer, Theta);
  //Innter Barrel
  E = protonSi.EvaluateInitialEnergy(E, 200 * micrometer, Theta);
  //LH2 target
  static TVector3 x1;
  x1 = x0 + dir;
  TVector3 T1(0,15,0);
  TVector3 T2(0,15,1);
  T1.SetPhi(dir.Phi());
  T2.SetPhi(dir.Phi());

  double d = 0;

  cout << "d=" << d << endl;
  cout << "Theta=" << Theta << endl;
  E = protonTarget.EvaluateInitialEnergy(E, d, Theta);
  cout << "Erec=" << E << endl;
  return E;
}

////////////////////////////////////////////////////////////////////////////////
TVector3 Analysis::InterpolateInPlaneZ(TVector3 V0, TVector3 V1, double Zproj) {
  TVector3 Vproj(-999, -999, -999);
  double t = (Zproj - V1.Z()) / (V1.Z() - V0.Z());
  double Xproj = V1.X() + (V1.X() - V0.X()) * t;
  double Yproj = V1.Y() + (V1.Y() - V0.Y()) * t;
  Vproj.SetXYZ(Xproj, Yproj, Zproj);
  return Vproj; 
}

////////////////////////////////////////////////////////////////////////////////
void Analysis::End(){
}
////////////////////////////////////////////////////////////////////////////////
void Analysis::InitOutputBranch(){
  RootOutput::getInstance()->GetTree()->Branch("Ex",&Ex, "Ex/D");
  RootOutput::getInstance()->GetTree()->Branch("E1",&E1, "E1/D");
  RootOutput::getInstance()->GetTree()->Branch("E2",&E2, "E2/D");
  RootOutput::getInstance()->GetTree()->Branch("Theta12",&Theta12, "Theta12/D");
  RootOutput::getInstance()->GetTree()->Branch("ThetaCM",&ThetaCM, "ThetaCM/D");
  RootOutput::getInstance()->GetTree()->Branch("VertexX",&VertexX, "VertexX/D");
  RootOutput::getInstance()->GetTree()->Branch("VertexY",&VertexY, "VertexY/D");
  RootOutput::getInstance()->GetTree()->Branch("VertexZ",&VertexZ, "VertexZ/D");
  RootOutput::getInstance()->GetTree()->Branch("VertexX_sim",&VertexX_sim, "VertexX_sim/D");
  RootOutput::getInstance()->GetTree()->Branch("VertexY_sim",&VertexY_sim, "VertexY_sim/D");
  RootOutput::getInstance()->GetTree()->Branch("VertexZ_sim",&VertexZ_sim, "VertexZ_sim/D");
  RootOutput::getInstance()->GetTree()->Branch("deltaX",&deltaX, "deltaX/D");
  RootOutput::getInstance()->GetTree()->Branch("deltaY",&deltaY, "deltaY/D");
  RootOutput::getInstance()->GetTree()->Branch("deltaZ",&deltaZ, "deltaZ/D");
  RootOutput::getInstance()->GetTree()->Branch("deltaPhi",&deltaPhi, "deltaPhi/D");
  RootOutput::getInstance()->GetTree()->Branch("sumTheta",&sumTheta, "sumTheta/D");

  RootOutput::getInstance()->GetTree()->Branch("alpha",&alpha, "alpha/D");

  RootOutput::getInstance()->GetTree()->Branch("Theta1",&Theta1, "Theta1/D");
  RootOutput::getInstance()->GetTree()->Branch("Phi1",&Phi1, "Phi1/D");
  RootOutput::getInstance()->GetTree()->Branch("Theta2",&Theta2, "Theta2/D");
  RootOutput::getInstance()->GetTree()->Branch("Phi2",&Phi2, "Phi2/D");
  RootOutput::getInstance()->GetTree()->Branch("Theta1s",&Theta1s, "Theta1s/D");
  RootOutput::getInstance()->GetTree()->Branch("Phi1s",&Phi1s, "Phi1s/D");
  RootOutput::getInstance()->GetTree()->Branch("Theta2s",&Theta2s, "Theta2s/D");
  RootOutput::getInstance()->GetTree()->Branch("Phi2s",&Phi2s, "Phi2s/D");
  RootOutput::getInstance()->GetTree()->Branch("TA",&TA, "TA/D");
  RootOutput::getInstance()->GetTree()->Branch("TAcalc",&TAcalc, "TAcalc/D");

  RootOutput::getInstance()->GetTree()->Branch("SiInnerZ1_X",&SiInnerZ1_X, "SiInnerZ1_X/D");
  RootOutput::getInstance()->GetTree()->Branch("SiInnerZ1_Y",&SiInnerZ1_Y, "SiInnerZ1_Y/D");
  RootOutput::getInstance()->GetTree()->Branch("SiInnerZ1_Z",&SiInnerZ1_Z, "SiInnerZ1_Z/D");
  RootOutput::getInstance()->GetTree()->Branch("SiOuterZ1_X",&SiOuterZ1_X, "SiOuterZ1_X/D");
  RootOutput::getInstance()->GetTree()->Branch("SiOuterZ1_Y",&SiOuterZ1_Y, "SiOuterZ1_Y/D");
  RootOutput::getInstance()->GetTree()->Branch("SiOuterZ1_Z",&SiOuterZ1_Z, "SiOuterZ1_Z/D");
  RootOutput::getInstance()->GetTree()->Branch("SiInnerZ1_X_sim",&SiInnerZ1_X_sim, "SiInnerZ1_X_sim/D");
  RootOutput::getInstance()->GetTree()->Branch("SiInnerZ1_Y_sim",&SiInnerZ1_Y_sim, "SiInnerZ1_Y_sim/D");
  RootOutput::getInstance()->GetTree()->Branch("SiInnerZ1_Z_sim",&SiInnerZ1_Z_sim, "SiInnerZ1_Z_sim/D");
  RootOutput::getInstance()->GetTree()->Branch("SiOuterZ1_X_sim",&SiOuterZ1_X_sim, "SiOuterZ1_X_sim/D");
  RootOutput::getInstance()->GetTree()->Branch("SiOuterZ1_Y_sim",&SiOuterZ1_Y_sim, "SiOuterZ1_Y_sim/D");
  RootOutput::getInstance()->GetTree()->Branch("SiOuterZ1_Z_sim",&SiOuterZ1_Z_sim, "SiOuterZ1_Z_sim/D");


  RootOutput::getInstance()->GetTree()->Branch("SiInnerX1_X",&SiInnerX1_X, "SiInnerX1_X/D");
  RootOutput::getInstance()->GetTree()->Branch("SiInnerX1_Y",&SiInnerX1_Y, "SiInnerX1_Y/D");
  RootOutput::getInstance()->GetTree()->Branch("SiInnerX1_Z",&SiInnerX1_Z, "SiInnerX1_Z/D");
  RootOutput::getInstance()->GetTree()->Branch("SiInnerX1_X_sim",&SiInnerX1_X_sim, "SiInnerX1_X_sim/D");
  RootOutput::getInstance()->GetTree()->Branch("SiInnerX1_Y_sim",&SiInnerX1_Y_sim, "SiInnerX1_Y_sim/D");
  RootOutput::getInstance()->GetTree()->Branch("SiInnerX1_Z_sim",&SiInnerX1_Z_sim, "SiInnerX1_Z_sim/D");

  RootOutput::getInstance()->GetTree()->Branch("theta_vertex_1_sim",&theta_vertex_1_sim, "theta_vertex_1_sim/D");
  RootOutput::getInstance()->GetTree()->Branch("theta_vertex_2_sim",&theta_vertex_2_sim, "theta_vertex_2_sim/D");

  RootOutput::getInstance()->GetTree()->Branch("InnerZ1_theta_sim",&InnerZ1_theta_sim, "&InnerZ1_theta_sim/D");
  RootOutput::getInstance()->GetTree()->Branch("InnerX1_theta_sim",&InnerX1_theta_sim, "&InnerX1_theta_sim/D");
  RootOutput::getInstance()->GetTree()->Branch("OuterZ1_theta_sim",&OuterZ1_theta_sim, "&OuterZ1_theta_sim/D");



  RootOutput::getInstance()->GetTree()->Branch("Distance",&Distance, "Distance/D");
  RootOutput::getInstance()->GetTree()->Branch("InteractionCoordinates","TInteractioncoordinates", &DC);
  RootOutput::getInstance()->GetTree()->Branch("ReactionConditions","ReactionConditions", &RC);
}
////////////////////////////////////////////////////////////////////////////////
void Analysis::InitInputBranch() {
  RootInput::getInstance()->GetChain()->SetBranchAddress("InteractionCoordinates",&DC);
  RootInput::getInstance()->GetChain()->SetBranchAddress("ReactionConditions",&RC);
}
////////////////////////////////////////////////////////////////////////////////
void Analysis::ReInitValue() {
  Ex = -1000;
  E1 = -1000;
  E2 = -1000;
  Theta12 = -1000;
  ThetaCM = -1000;
  VertexX = -1000;
  VertexY = -1000;
  VertexZ = -1000;
  VertexX_sim = -1000;
  VertexY_sim = -1000;
  VertexZ_sim = -1000;
  deltaX = -1000;
  deltaY = -1000;
  deltaZ = -1000;
  Distance = -1000;
  sumTheta = -1000;
  deltaPhi = -1000;
  alpha = -1000;
  Theta1 = -1000;
  Phi1 = -1000;
  Theta2 = -1000;
  Phi2 = -1000;
  Theta1s = -1000;
  Phi1s = -1000;
  Theta2s = -1000;
  Phi2s = -1000;
  TA = -1000;
  TAcalc = -1000;

  z_1_sim = -1000;
  z_2_sim = -1000;
  z_3_sim = -1000;
  z_4_sim = -1000;

  theta_vertex_1_sim = -1000;
  theta_vertex_2_sim = -1000;

  SiInnerZ1_X = -1000;
  SiInnerZ1_Y = -1000;
  SiInnerZ1_Z = -1000;
  SiInnerZ1_X_sim = -1000;
  SiInnerZ1_Y_sim = -1000;
  SiInnerZ1_Z_sim = -1000;
  SiOuterZ1_X = -1000;
  SiOuterZ1_Y = -1000;
  SiOuterZ1_Z = -1000;
  SiOuterZ1_X_sim = -1000;
  SiOuterZ1_Y_sim = -1000;
  SiOuterZ1_Z_sim = -1000;

  SiInnerX1_X = -1000;
  SiInnerX1_Y = -1000;
  SiInnerX1_Z = -1000;
  SiInnerX1_X_sim = -1000;
  SiInnerX1_Y_sim = -1000;
  SiInnerX1_Z_sim = -1000;
  
  InnerZ1_theta_sim = -1000;
  InnerX1_theta_sim = -1000;
  OuterZ1_theta_sim = -1000;

}	

////////////////////////////////////////////////////////////////////////////////
//            Construct Method to be pass to the DetectorFactory              //
////////////////////////////////////////////////////////////////////////////////
NPL::VAnalysis* Analysis::Construct(){
  return (NPL::VAnalysis*) new Analysis();
}

////////////////////////////////////////////////////////////////////////////////
//            Registering the construct method to the factory                 //
////////////////////////////////////////////////////////////////////////////////
extern "C"{
class proxy{
  public:
    proxy(){
      NPL::AnalysisFactory::getInstance()->SetConstructor(Analysis::Construct);
    }
};

proxy p;
}

