#include <vector> // Needed to store histograms.

void HistoPosFiller(int RunStart, int RunStop) {
  // **************************************************************************
  // This macro simply fill an histogram per FPMW and store them in a root file.
  // It is needed for the input of the LinearisationYaml.cxx macro.
  //
  // Input :
  //    linea_%d : A root file containing the reconstructed position of the
  //    FPMW. l24
  //
  //    RunStart/Stop : Number of the starting/stop run
  //
  //
  // Output:
  //    HistUnlinea.root : The histograms of reconstructed position l116
  // **************************************************************************

  //========================================================================
  //-------------------Open tree and define Hist----------------------------
  //========================================================================

  // Import the chain
  TChain *ch = new TChain("PhysicsTree");
  for (int run = RunStart; run <= RunStop; run++) {
    ch->Add(Form("../../root/analysis/linea_%d.root", run));
  } // End of run loop

  // Declaration of the histograms storing the reconstructed position
  vector<TH1F *> Position;
  for (int i = 1; i <= 4; i++) {
    std::string PosXName = "hPositionXDet" + std::to_string(i);
    std::string PosXTitle = "PositionXDet" + std::to_string(i);
    std::string PosYName = "hPositionYDet" + std::to_string(i);
    std::string PosYTitle = "PositionYDet" + std::to_string(i);
    TH1F *PosX;
    TH1F *PosY;

    // Be wary here, the code is very sensible to the number of bin
    // and to the span of the histograms. One must be sure that when
    // executiong the LinearisationYaml.cxx there is no error.

    int LowBin = -90, Highbin = 90;
    int Nbin = (Highbin - LowBin) * 10;
    int LowBin2 = -1000, Highbin2 = 1000;
    int Nbin2 = (Highbin2 - LowBin2) * 10;

    if (i == 1) {
      PosX =
          new TH1F(PosXName.c_str(), PosXTitle.c_str(), Nbin, LowBin, Highbin);
      PosY =
          new TH1F(PosYName.c_str(), PosYTitle.c_str(), Nbin, LowBin, Highbin);
    } // End if det 1

    if (i == 2) {
      PosX =
          new TH1F(PosXName.c_str(), PosXTitle.c_str(), Nbin, LowBin, Highbin);
      PosY =
          new TH1F(PosYName.c_str(), PosYTitle.c_str(), Nbin, LowBin, Highbin);

    } // End if det 2

    if (i == 3 || i == 4) {
      PosX = new TH1F(PosXName.c_str(), PosXTitle.c_str(), Nbin2, LowBin2,
                      Highbin2);
      PosY =
          new TH1F(PosYName.c_str(), PosYTitle.c_str(), Nbin, LowBin, Highbin);
    } // End if det 3 or 4

    Position.push_back(PosX);
    Position.push_back(PosY);
  } // End of i loop

  //========================================================================
  //-----------------------Importing FPMW-----------------------------------
  //========================================================================

  TFPMWPhysics *FPMW = new TFPMWPhysics();
  ch->SetBranchAddress("FPMW", &FPMW);

  //========================================================================
  //---------------------------Treat Event----------------------------------
  //========================================================================
  int Nentries = ch->GetEntries();
  int GoodEntries = 0;

  for (int entries = 0; entries < Nentries; entries++) {
    ch->GetEntry(entries);
    if (entries % 100000 == 0) {
      cout << "\033[92m\r Processing tree..."
           << (double)entries / Nentries * 100 << "\% done" << flush;
    } // End cout loading

    // Verification that we are not on a empty event for the FPMW
    if (FPMW->DetectorNbr.size() > 0) {
      GoodEntries += 1;

      // Parsing through the different FPMW
      for (int DetectorNbr = 0; DetectorNbr < FPMW->DetectorNbr.size();
           DetectorNbr++) {

        double X = FPMW->PositionX[DetectorNbr];
        double Y = FPMW->PositionY[DetectorNbr];

        if (DetectorNbr >= 0 && DetectorNbr < 4) {
          Position[DetectorNbr * 2]->Fill(X);
          Position[DetectorNbr * 2 + 1]->Fill(Y);
        } // end fill histo
      }   // Loop detnbr
    }     // Non empty event
  }       // End loop on entries

  cout << endl << "Good entries" << GoodEntries << endl;

  //========================================================================
  //---------------------Output histogram-----------------------------------
  //========================================================================
  TFile *OutputHist = new TFile("HistUnlinea.root", "RECREATE");
  std::vector<TCanvas *> canvas, cLinea;

  for (int i = 0; i < Position.size(); i++) {

    // Histo name
    std::string CanvasName = "c" + std::to_string(i);
    std::string CanvasNameLinea = "Linearised" + std::to_string(i);

    TCanvas *c = new TCanvas(CanvasName.c_str(), CanvasName.c_str(), 960, 540);
    canvas.push_back(c);

    canvas[i]->cd(1);
    if (true) {
      Position[i]->Draw("logY");
    }
    Position[i]->Write();
    gPad->SetLogy();
    gStyle->SetOptStat(11111111);
  }
  OutputHist->Close();
} // end macro
