TGraphErrors* GetYield(string);
TGraph* GetGEFYield(int ZCN, int ACN);
void Draw(int A, string isotope, TGraphErrors* ge, TGraph* g2, double Ymax, double scale);

vector<int> vZ;
vector<int> vA;
vector<int> vN;
vector<string> visoname;
vector<string> yield_file;

int NumberOfZ=0;
int NumberOfN=0;

////////////////////////////////////////////
void MapYield()
{
  gROOT->SetStyle("pierre_style");

  //string filename = "list_nature.dat";
  //string filename = "list_Hg.dat";
  string filename = "list_divers.dat";
  ifstream ifile;
  ifile.open(filename.c_str());

  int Z=0;
  int Zprev=0;
  int A;
  NPL::Particle* iso;
  while(ifile>>Z>>A){
    //ifile >> Z >> A;
    iso = new NPL::Particle(Z,A);
    string isoname = iso->GetName();
    isoname.resize(5);
    string yield_filename = "dat/yield_" + isoname + ".dat";
    yield_file.push_back(yield_filename);
    cout << yield_filename << endl;
    if(Z!=Zprev){
      Zprev = Z;
      NumberOfZ++;
    }
    vZ.push_back(Z);
    vA.push_back(A);
    vN.push_back(A-Z);
    visoname.push_back(isoname);
  }
  NumberOfN = 1;//vN[vN.size()-1] - vN[0] + 1;
  cout << "***** " << NumberOfZ << " different element to be plotted !" << endl; 
  cout << "***** " << NumberOfN << " number of neutrons" << endl; 
  cout << "size " << vZ.size() << " " << NumberOfZ*NumberOfN << endl;

  //TCanvas* c1 = new TCanvas("Map1","Map1",3500,4000);
  TCanvas* c1 = new TCanvas("Map1","Map1",2000,2000);
    
  //c1->Divide(NumberOfN,NumberOfZ,0,0);
  c1->Divide(2,3,0);
  //c1->Divide(7,2);
  c1->Modified();

  int Nmin = vN[0];
  int Zmin = vZ[vZ.size()-1];
  int Zmax = vZ[0];
  int Nmax = vN[vN.size()-1];
  int indice = 0;
  int i0 = 0;
  int i1 = 0;
  int i2 = 0;
  int i3 = 0;
  int i4 = 0;
  for(int i=0; i<vZ.size(); i++){
    int DeltaN = vN[i] - Nmin;
    int DeltaZmax = abs(vZ[i] - Zmax);
    int DeltaZmin = abs(vZ[i] - Zmin);
    //int indice = DeltaN + NumberOfN*DeltaZ;
    if(DeltaZmax==0){
      indice = i0 + NumberOfN*DeltaZmax;
      i0++;
    }
    if(DeltaZmax==1){
      indice = i1 + NumberOfN*DeltaZmax;
      i1++;
    }
    if(DeltaZmax==2){
      indice = i2 + NumberOfN*DeltaZmax;
      i2++;
    }
    if(DeltaZmax==3){
      indice = i3 + NumberOfN*DeltaZmax;
      i3++;
    }
     if(DeltaZmax==4){
      indice = i4 + NumberOfN*DeltaZmax;
      i4++;
    }
 
    cout << vZ[i] << " " << vA[i] << " " << i << " " << indice << endl;
    c1->cd(indice+1+1);


    TGraphErrors* gdata = GetYield(yield_file[i]);
    TGraph* ggef = GetGEFYield(vZ[i],vA[i]);
    string isoname = visoname[i].substr(3);
    double scale = 1;
    if(indice+2 !=5 && indice+2 !=6) scale = 1.17;
    if(vZ[i]==81) Draw(vA[i],isoname,gdata,ggef,26,scale);
    else Draw(vA[i],isoname,gdata,ggef,26,scale);
  }
  ifile.close();

  vZ.clear();
  vA.clear();
  vN.clear();
  yield_file.clear();
  NumberOfN=0;
  NumberOfZ=0;

}

////////////////////////////////////////////
void Draw(int A, string isoname, TGraphErrors* ge, TGraph* g2, double Ymax, double scale){
  string isotope = "^{" + to_string(A) + "}" + isoname;
  TLatex* iso = new TLatex(44,Ymax-6,isotope.c_str());
  
  TH2F* h2 = new TH2F("h2","h2",500,25,55,500,0.1,28);
  h2->GetXaxis()->SetTitle("Proton Number Z");
  h2->GetYaxis()->SetTitle("Y(Z) (%)");
  h2->GetXaxis()->CenterTitle();
  h2->GetYaxis()->CenterTitle();

  h2->GetXaxis()->SetLabelSize(0.1*scale);
  h2->GetXaxis()->SetTitleSize(0.11*scale);
  h2->GetYaxis()->SetLabelSize(0.11*scale);
  h2->GetYaxis()->SetTitleSize(0.11*scale);
  h2->GetXaxis()->SetTitleOffset(0.75);
  h2->GetYaxis()->SetTitleOffset(0.65*1./scale);
  h2->GetYaxis()->SetNdivisions(505);

  iso->SetTextFont(42);
  iso->SetTextSize(0.15*scale);

  g2->SetMarkerColor(2);
  g2->SetLineColor(2);
  g2->SetMarkerStyle(23);
  g2->SetMarkerSize(2);
  g2->SetLineWidth(3);

  h2->GetXaxis()->SetRangeUser(27,53);
  h2->GetYaxis()->SetRangeUser(0,Ymax);
  h2->Draw();
  iso->Draw();
  ge->SetMarkerSize(2);
  ge->SetFillColor(kAzure-6);
  ge->Draw("plfsame");
  ge->Draw("psame");
  g2->Draw("plsame");

  TLine* line = new TLine(36,0,36,26);
  line->SetLineWidth(5);
  line->SetLineStyle(10);
  line->SetLineColor(kGreen-2);
  line->Draw("lsame");
}

////////////////////////////////////////////
TGraph* GetGEFYield(int ZCN, int ACN){
  TGraph* g1 = new TGraph();

  string pathname = "/home/sofia/Physics/Codes/GEF_2023.1.2/Yields/";
  string filename = pathname + "GEF2023_YieldZ_" + to_string(ZCN) + "_" + to_string(ACN) + ".txt";

  cout << "Reading GEF file " << filename << endl;
  ifstream ifile;
  ifile.open(filename.c_str());

  int Z;
  double Y;
  int i=0;
  while(ifile>>Z>>Y){
    g1->SetPoint(i,Z,Y);
    i++;
  }
  ifile.close();

  return g1;
}

////////////////////////////////////////////
TGraphErrors* GetYield(string filename)
{
  ifstream ifile;

  ifile.open(filename.c_str());

  int Z;
  double yield;
  double yield_err;
  TGraphErrors* g = new TGraphErrors();
  int i=0;
  while(!ifile.eof()){
    ifile >> Z >> yield >> yield_err;
    //cout << Z << " " << yield << " " << yield_err << endl;
    //if(yield>0 && yield_err>0){
      g->SetPoint(i,Z,yield);
      g->SetPointError(i,0,yield_err);
      i++;
    //}
  }

  g->SetMarkerStyle(8);

  ifile.close();
  return g;
}


